package com.example.xavierferraronsserra.sobreruedas.Adapters;

/**
 * Created by xavierferraronsserra on 24/4/18.
 */

import android.content.Context;
import android.graphics.Color;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import com.example.xavierferraronsserra.sobreruedas.Model.Anuncio;
import com.example.xavierferraronsserra.sobreruedas.R;
import com.example.xavierferraronsserra.sobreruedas.Utils.Utils;
import com.google.firebase.firestore.DocumentSnapshot;
import com.google.firebase.firestore.Query;
import com.makeramen.roundedimageview.RoundedTransformationBuilder;
import com.squareup.picasso.Picasso;
import com.squareup.picasso.Transformation;

import butterknife.BindView;
import butterknife.ButterKnife;

/**
 * Created by xavierferraronsserra on 12/4/18.
 */

public class AnunciosUsuarioAdapter extends FirestoreAdapter<AnunciosUsuarioAdapter.AnunciosHolder> {

    private AnunciosHolder.ClickListener mClickListener;
    private Context mContext;

    public AnunciosUsuarioAdapter(Context context, Query query, AnunciosHolder.ClickListener clickListener, FinishLoading finishLoading) {
        super(query,finishLoading);
        mClickListener=clickListener;
        mContext= context;
    }

    @Override
    public AnunciosHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        LayoutInflater inflater = LayoutInflater.from(parent.getContext());
        return new AnunciosHolder(inflater.inflate(R.layout.list_item_anuncio_usuario,parent,false), mClickListener);
    }

    @Override
    public void onBindViewHolder(AnunciosHolder holder, int position) {
        if(isSelected(position)){
            holder.itemView.setBackgroundColor(Color.LTGRAY);
        }
        else{
            holder.itemView.setBackgroundColor(Color.WHITE);
        }
        holder.bind(mContext, getSnapshot(position));
    }

    public static class AnunciosHolder extends RecyclerView.ViewHolder implements View.OnClickListener, View.OnLongClickListener {

        private ClickListener mClickListener;
        private Anuncio mAnuncio;
        private Context mContext;

        @BindView(R.id.list_item_anuncio_usuario_image) ImageView mImageView;
        @BindView(R.id.list_item_anuncio_usuario_title)  TextView mTextViewTitle;
        @BindView(R.id.list_item_anuncio_usuario_price)  TextView mTextViewPrice;
        @BindView(R.id.list_item_anuncio_usuario_date)  TextView mTextViewDate;
        @BindView(R.id.list_item_anuncio_usuario_state)  TextView mTextViewState;



        public AnunciosHolder(View itemView, ClickListener clickListener) {
            super(itemView);
            mClickListener = clickListener;
            ButterKnife.bind(this, itemView);
            itemView.setOnClickListener(this);
            itemView.setOnLongClickListener(this);

        }

        public void bind(Context context,DocumentSnapshot snapshot){
            mContext = context;
            mAnuncio = snapshot.toObject(Anuncio.class);
            Transformation transformation = new RoundedTransformationBuilder()
                    .cornerRadiusDp(40)
                    .oval(false)
                    .build();
            if(mAnuncio.getFotos().size()!=0) {
                Picasso.get().load(mAnuncio.getFotoByPosition(String.valueOf(0)).getUrl()).fit().centerCrop().placeholder(R.drawable.progress_animation).error(R.drawable.ic_error).noFade().into(mImageView);
            }
            mTextViewTitle.setText(mAnuncio.getTitle());
            mTextViewPrice.setText(Utils.formatPrice(mAnuncio.getPrice()));
            mTextViewDate.setText(Utils.formatDate(mAnuncio.getFecha_renovacion()));
            if(mAnuncio.isActivo()){
                mTextViewState.setText(mContext.getResources().getString(R.string.active));
                mTextViewState.setTextColor(mContext.getResources().getColor(R.color.colorGreen));
            }
            else{
                mTextViewState.setText(mContext.getResources().getString(R.string.caduced));
                mTextViewState.setTextColor(mContext.getResources().getColor(R.color.colorRed));
            }
        }

        @Override
        public void onClick(View v) {
            if (mClickListener!=null) {
                mClickListener.onItemClicked(getAdapterPosition(),mAnuncio);
            }
        }

        @Override
        public boolean onLongClick(View v) {
            if(mClickListener!=null){
                return mClickListener.onItemLongClicked(getAdapterPosition());
            }
            return false;
        }

        public interface ClickListener {
            public void onItemClicked(int position, Anuncio anuncio);
            public boolean onItemLongClicked(int position);

        }
    }
}
