package com.example.xavierferraronsserra.sobreruedas.Adapters;


import android.support.v7.widget.RecyclerView;
import android.util.Log;

import com.example.xavierferraronsserra.sobreruedas.Model.Anuncio;
import com.google.firebase.firestore.DocumentChange;
import com.google.firebase.firestore.DocumentSnapshot;
import com.google.firebase.firestore.EventListener;
import com.google.firebase.firestore.FirebaseFirestoreException;
import com.google.firebase.firestore.ListenerRegistration;
import com.google.firebase.firestore.Query;
import com.google.firebase.firestore.QuerySnapshot;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;


public abstract class FirestoreAdapter<VH extends RecyclerView.ViewHolder>
        extends RecyclerView.Adapter<VH>
        implements EventListener<QuerySnapshot> {

    private final String TAG = "FirestoreAdapter";
    private ArrayList<DocumentSnapshot> mSnapshots = new ArrayList<>();
    private HashMap<String, Boolean> mSelectedItems = new HashMap<String,Boolean>();

    private Query mQuery;
    private FinishLoading mFinishLoading;
    private ListenerRegistration mRegistration;
    private boolean mMultiselection = false;



    public FirestoreAdapter(Query query, FinishLoading finishLoading) {
        mQuery = query;
        mFinishLoading=finishLoading;
    }


    @Override
    public void onEvent(QuerySnapshot documentSnapshots,
                        FirebaseFirestoreException e) {

        // Handle errors
        if (e != null) {
            Log.w(TAG, "onEvent:error", e);
            return;
        }

        // Dispatch the event
        for (DocumentChange change : documentSnapshots.getDocumentChanges()) {
            // Snapshot of the changed document
            DocumentSnapshot snapshot = change.getDocument();

            switch (change.getType()) {
                case ADDED:
                    onDocumentAdded(change);
                    break;
                case MODIFIED:
                    onDocumentModified(change);
                    break;
                case REMOVED:
                    onDocumentRemoved(change);
                    break;
            }
        }
        mFinishLoading.finishloading();
        onDataChanged();
    }

    protected void onDocumentAdded(DocumentChange change) {
        mSnapshots.add(change.getNewIndex(), change.getDocument());
        notifyItemInserted(change.getNewIndex());
    }

    protected void onDocumentModified(DocumentChange change) {
        if (change.getOldIndex() == change.getNewIndex()) {
            // Item changed but remained in same position
            mSnapshots.set(change.getOldIndex(), change.getDocument());
            notifyItemChanged(change.getOldIndex());
        } else {
            // Item changed and changed position
            mSnapshots.remove(change.getOldIndex());
            mSnapshots.add(change.getNewIndex(), change.getDocument());
            notifyItemMoved(change.getOldIndex(), change.getNewIndex());
        }
    }

    protected void onDocumentRemoved(DocumentChange change) {
        mSnapshots.remove(change.getOldIndex());
        notifyItemRemoved(change.getOldIndex());
    }

    public void startListening() {
        if (mQuery != null && mRegistration == null) {
            mRegistration = mQuery.addSnapshotListener(this);
        }
    }

    public void stopListening() {
        if (mRegistration != null) {
            mRegistration.remove();
            mRegistration = null;
        }

        mSnapshots.clear();
        notifyDataSetChanged();
    }

    public void setQuery(Query query) {
        // Stop listening
        stopListening();

        // Clear existinkodig data
        mSnapshots.clear();
        notifyDataSetChanged();

        // Listen to new query
        mQuery = query;
        startListening();
    }

    @Override
    public int getItemCount() {
        return mSnapshots.size();
    }

    protected DocumentSnapshot getSnapshot(int index) {
        return mSnapshots.get(index);
    }

    protected void onError(FirebaseFirestoreException e) {};

    protected void onDataChanged() {}

    /**
     * Indicates if the item at position position is selected
     *
     * @param position Position of the item to check
     * @return true if the item is selected, false otherwise
     */
    public boolean isSelected(int position) {
        String anuncioId=mSnapshots.get(position).toObject(Anuncio.class).getId();
        if( getSelectedItems().get(anuncioId)!=null){
            return true;
        }
        else return false;
    }

    /**
     * Toggle the selection status of the item at a given position
     *
     * @param position Position of the item to toggle the selection status for
     */
    public void toggleSelection(int position) {
        String key = mSnapshots.get(position).getId();
        if(mSelectedItems.get(key)!=null){
            mSelectedItems.remove(key);
        } else {
            mSelectedItems.put(key, mSnapshots.get(position).toObject(Anuncio.class).isActivo());
        }
        if (mSelectedItems.size() == 0) {
            setMultiselection(false);
        }
        notifyItemChanged(position);
    }

    /**
     * Clear the selection status for all items
     */
    public void clearSelection() {
        HashMap<String,Boolean> selection = getSelectedItems();
        Iterator<String> it= selection.keySet().iterator();
        while(it.hasNext()){
            String key= (String) it.next();
            for(int j=0; j<mSnapshots.size();j++) {
                if (mSnapshots.get(j).getId().equals(key)) {
                    notifyItemChanged(j);
                }
            }
        }
            mSelectedItems.clear();
            mMultiselection = false;
    }

    /**
     * Count the selected items
     *
     * @return Selected items count
     */
    public int getSelectedItemCount() {
        return mSelectedItems.size();
    }

    /**
     * Indicates the list of selected items
     *
     * @return List of selected items ids
     */
    public HashMap<String,Boolean> getSelectedItems() {
        List<String> items = new ArrayList<>(mSelectedItems.size());
        Iterator<Map.Entry<String, Boolean>> entries = mSelectedItems.entrySet().iterator();
        while (entries.hasNext()) {
            Map.Entry<String, Boolean> entry = entries.next();
            items.add(entry.getKey());
        }
        return mSelectedItems;
    }

    public boolean isMultiselection() {
        return mMultiselection;
    }

    public void setMultiselection(boolean multiselection) {
        mMultiselection = multiselection;
    }

    public interface FinishLoading{
        public boolean finishloading();
    }

}