package com.example.xavierferraronsserra.sobreruedas;

import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.content.pm.ResolveInfo;
import android.net.Uri;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.support.v4.view.ViewPager;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;
import android.widget.Toast;

import com.example.xavierferraronsserra.sobreruedas.Adapters.SlidingImageAdapter;
import com.example.xavierferraronsserra.sobreruedas.Model.Anuncio;
import com.example.xavierferraronsserra.sobreruedas.Model.Usuario;
import com.example.xavierferraronsserra.sobreruedas.Utils.StorageManagement;
import com.example.xavierferraronsserra.sobreruedas.Utils.Utils;
import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.OnMapReadyCallback;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.MarkerOptions;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.firestore.DocumentSnapshot;
import com.viewpagerindicator.CirclePageIndicator;

import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;

/**
 * Created by xavierferraronsserra on 13/4/18.
 */

public class DetalleAnuncioFragment extends Fragment implements OnMapReadyCallback {

    public static final String ARG_ANUNCIO = "com.example.xavierferraronsserra.sobreruedas.anuncio";

    private Anuncio mAnuncio;
    private SlidingImageAdapter adapter;
    private Usuario mUsuario;

    private GoogleMap mMap;
    private SupportMapFragment mMapFragment;
    private final int DEFAULT_ZOOM = 14;

    @BindView(R.id.detalle_anuncio_pager)
    ViewPager mViewPager;
    @BindView(R.id.detalle_anuncio_indicator)
    CirclePageIndicator mCirclePageIndicator;
    @BindView(R.id.detalle_anuncio_title)
    TextView mTextViewTitle;
    @BindView(R.id.detalle_anuncio_price)
    TextView mTextViewPrice;
    @BindView(R.id.detalle_anuncio_visits)
    TextView mTextViewVisits;
    @BindView(R.id.detalle_anuncio_description)
    TextView mTextViewDescription;

    public static DetalleAnuncioFragment newInstance(Anuncio anuncio) {
        Bundle args = new Bundle();
        args.putParcelable(ARG_ANUNCIO, anuncio);
        DetalleAnuncioFragment detalleAnuncioFragment = new DetalleAnuncioFragment();
        detalleAnuncioFragment.setArguments(args);
        return detalleAnuncioFragment;
    }

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setHasOptionsMenu(true);
        mAnuncio = (Anuncio) getArguments().getParcelable(ARG_ANUNCIO);
        StorageManagement.getUser(getContext(), mAnuncio.getId_usuario()).get()
                .addOnCompleteListener(new OnCompleteListener<DocumentSnapshot>() {
                    @Override
                    public void onComplete(@NonNull Task<DocumentSnapshot> task) {
                        if (task.isSuccessful()) {
                            mUsuario = task.getResult().toObject(Usuario.class);
                        }
                    }
                });

        StorageManagement.incrementVisits(mAnuncio.getId());
    }

    @Nullable
    @Override
    public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_detalle_anuncio, container, false);
        ButterKnife.bind(this, view);

        adapter = new SlidingImageAdapter(getContext(), mAnuncio);
        mViewPager.setAdapter(adapter);
        mCirclePageIndicator.setViewPager(mViewPager);

        final float density = getResources().getDisplayMetrics().density;
        mCirclePageIndicator.setRadius(5 * density);

        mTextViewTitle.setText(mAnuncio.getTitle());
        mTextViewPrice.setText(Utils.formatPrice(mAnuncio.getPrice()));
        mTextViewDescription.setText(mAnuncio.getDescripcion());
        mTextViewVisits.setText(Integer.toString(mAnuncio.getVisitas()));

        return view;
    }

    @Override
    public void onActivityCreated(@Nullable Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);

        mMapFragment = (SupportMapFragment) getChildFragmentManager().findFragmentById(R.id.detalle_anuncio_map);
        mMapFragment.getMapAsync(this);
    }

    @Override
    public void onMapReady(GoogleMap googleMap) {
        mMap = googleMap;
        updateUI();
    }

    private void updateUI() {
        updateZoomUI();
        addMarker();
    }

    private void updateZoomUI() {
        if (mMap == null) {
            return;
        }
        mMap.getUiSettings().setZoomControlsEnabled(true);
    }

    private void addMarker() {
        mMap.clear();
        MarkerOptions markerOptions = new MarkerOptions();
        LatLng latLng = new LatLng(mAnuncio.getPosicion().getLatitude(), mAnuncio.getPosicion().getLongitude());
        markerOptions.position(latLng);
        markerOptions.title(mAnuncio.getAddress());
        mMap.addMarker(markerOptions).showInfoWindow();
        mMap.moveCamera(CameraUpdateFactory.newLatLngZoom(latLng, DEFAULT_ZOOM));
    }


    @OnClick(R.id.detalle_anuncio_phone)
    public void phone() {
        if (mUsuario != null && mUsuario.isContactoTelefono()) {
            makePhoneCall(mUsuario.getTelefono());
        }
        else {
            Toast.makeText(getContext(), getResources().getString(R.string.validation_contact_form_active), Toast.LENGTH_SHORT).show();
        }
    }

    private void makePhoneCall(String tel) {
        Intent phoneCall = new Intent(Intent.ACTION_DIAL);
        phoneCall.setData(Uri.parse("tel:" + tel));

        if (isAvailable(getContext(), phoneCall)) {
            startActivity(phoneCall);
        } else {
            Toast.makeText(getContext(), getResources().getString(R.string.validation_available), Toast.LENGTH_SHORT).show();
        }
        if (!mUsuario.isContactoTelefono()) {
            Toast.makeText(getContext(), getResources().getString(R.string.validation_contact_form_active), Toast.LENGTH_SHORT).show();
        }
    }

    public static boolean isAvailable(Context ctx, Intent intent) {
        final PackageManager mgr = ctx.getPackageManager();
        List<ResolveInfo> list =
                mgr.queryIntentActivities(intent,
                        PackageManager.MATCH_DEFAULT_ONLY);
        return list.size() > 0;
    }

    @OnClick(R.id.detalle_anuncio_email)
    public void email() {
        if (mUsuario != null && mUsuario.isContactoEmail()) {
            sendEmail(mUsuario.getEmail());
        } else {
            Toast.makeText(getContext(), getResources().getString(R.string.validation_contact_form_active), Toast.LENGTH_SHORT).show();
        }
    }

    private void sendEmail(String email) {
        Intent emailIntent = new Intent(Intent.ACTION_SENDTO);
        emailIntent.setData(Uri.parse("mailto:"));
        emailIntent.putExtra(Intent.EXTRA_EMAIL, new String[]{mUsuario.getEmail()});
        emailIntent.putExtra(Intent.EXTRA_SUBJECT, getResources().getString(R.string.email_title) + mAnuncio.getTitle());

        if (isAvailable(getContext(), emailIntent)) {

            startActivity(emailIntent);
        }

    }

    @OnClick(R.id.detalle_anuncio_xat)
    public void chat() {
        sendChat(mAnuncio.getId_usuario(), mAnuncio.getId());
    }

    public void sendChat(String userId, String anuncioId) {
        Toast.makeText(getContext(), getResources().getString(R.string.validation_not_implemented), Toast.LENGTH_SHORT).show();

    }


}
