package com.example.xavierferraronsserra.sobreruedas;

import android.*;
import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.location.Location;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AlertDialog;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ProgressBar;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import com.example.xavierferraronsserra.sobreruedas.Model.Agrupation;
import com.example.xavierferraronsserra.sobreruedas.Model.Filter;
import com.example.xavierferraronsserra.sobreruedas.Utils.StorageManagement;
import com.example.xavierferraronsserra.sobreruedas.Utils.Utils;
import com.google.android.gms.location.FusedLocationProviderClient;
import com.google.android.gms.location.LocationCallback;
import com.google.android.gms.location.LocationRequest;
import com.google.android.gms.location.LocationResult;
import com.google.android.gms.location.LocationServices;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.firestore.DocumentSnapshot;
import com.google.firebase.firestore.QuerySnapshot;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import java.util.ArrayList;
import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;
import io.apptik.widget.MultiSlider;

/**
 * Created by xavierferraronsserra on 27/4/18.
 */

public class FilterFragment extends android.support.v4.app.Fragment {

    private final String TAG = "FilterFragment";
    public static final String EXTRA_FILTER = "com.example.xavierferraronsserra.sobreruedas.filter";
    public static final String PREFERENCES_FILTER = "com.example.xavierferraronsserra.sobreruedas.filter_preferences";
    public static final String PREFERENCES_KEY = "filter";

    private final int REQUEST_ACCESS_FINE_LOCATION = 1;
    private boolean mLocationPermissionGranted = false;
    private FusedLocationProviderClient mFusedLocationProviderClient;
    private Location mLastKnownLocation;
    private Handler mHandler;

    private List<Agrupation> mAgrupationList;
    private Filter mFilter;

    private final static int MAX_DISTANCE = 250;
    private final static int MAX_PRICE = 10000;

    MultiSlider.Thumb mDistanceThumb;
    MultiSlider.Thumb mPriceInfThumb;
    MultiSlider.Thumb mPriceSupThumb;

    @BindView(R.id.filter_search_text)
    TextView mTextViewSearch;
    @BindView(R.id.filter_agrupations)
    Spinner mAgrupationsSpinner;
    @BindView(R.id.filter_slider_distance)
    MultiSlider mSliderDistance;
    @BindView(R.id.filter_slider_price)
    MultiSlider mSliderPrice;
    @BindView(R.id.filter_min_price)
    TextView mTextViewMinPrice;
    @BindView(R.id.filter_max_price)
    TextView mTextViewMaxPrice;
    @BindView(R.id.filter_max_distance)
    TextView mTextViewMaxDistance;
    @BindView(R.id.filter_apply)
    Button mButtonApply;
    @BindView(R.id.filter_progress_bar)
    ProgressBar mProgressBar;

    public static Filter newFilter(){
        //Creates a filter with no limits
        return new Filter(MAX_PRICE,  MAX_DISTANCE);
    }

    public static int getMaxDistance() {
        return MAX_DISTANCE;
    }

    public static int getMaxPrice() {
        return MAX_PRICE;
    }

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setHasOptionsMenu(true);
        mHandler=new Handler();
        mFusedLocationProviderClient = LocationServices.getFusedLocationProviderClient(getContext());
    }

    @Nullable
    @Override
    public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_filter, container, false);
        ButterKnife.bind(this, view);
        mProgressBar.setVisibility(View.VISIBLE);
        getActivity().getWindow().setFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE, WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);

        //Gets agrupation descriptions to show as filter options
        StorageManagement.getAgrupations(getContext())
                .get()
                .addOnCompleteListener(new OnCompleteListener<QuerySnapshot>() {
                    @Override
                    public void onComplete(@NonNull Task<QuerySnapshot> task) {
                        if (task.isSuccessful()) {
                            mAgrupationList = new ArrayList<>();
                            mAgrupationList.add(new Agrupation("0", "Seleccione una agrupación"));
                            for (DocumentSnapshot document : task.getResult()) {
                                Log.d(TAG, document.getId() + " => " + document.toObject(Agrupation.class).toString());
                                Agrupation agrupation = document.toObject(Agrupation.class);
                                agrupation.setId(document.getId());
                                mAgrupationList.add(agrupation);
                            }
                            ArrayAdapter<CharSequence> adapter = new ArrayAdapter(getActivity(), R.layout.spinner_item, mAgrupationList);
                            adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
                            mAgrupationsSpinner.setAdapter(adapter);

                            mSliderPrice.setMax(MAX_PRICE);
                            mPriceInfThumb = mSliderPrice.getThumb(0);
                            mPriceSupThumb = mSliderPrice.getThumb(1);

                            mSliderDistance.setMax(MAX_DISTANCE);
                            mDistanceThumb = mSliderDistance.getThumb(0);

                            SharedPreferences preferences = getActivity().getSharedPreferences(PREFERENCES_FILTER, Context.MODE_PRIVATE);
                            if (preferences.contains(PREFERENCES_KEY)) {
                                GsonBuilder builder = new GsonBuilder();
                                Gson gson = builder.create();
                                mFilter = gson.fromJson(preferences.getString(PREFERENCES_KEY, ""), Filter.class);
                            } else {
                                initFilter();
                            }
                            refreshUI();


                        } else {
                            Log.d(TAG, "Error getting documents: ", task.getException());
                        }
                        mProgressBar.setVisibility(View.GONE);
                        getActivity().getWindow().clearFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);


                    }
                });


        mSliderDistance.setOnThumbValueChangeListener(new MultiSlider.OnThumbValueChangeListener() {
            @Override
            public void onValueChanged(MultiSlider multiSlider, MultiSlider.Thumb thumb, int thumbIndex, int value) {
                if (value < MAX_DISTANCE) {
                    mTextViewMaxDistance.setText(String.valueOf(Utils.formatDistance(value)));
                } else {
                    mTextViewMaxDistance.setText(getResources().getString(R.string.more_than) + String.valueOf(Utils.formatDistance(value)));
                }
            }
        });

        mSliderPrice.setOnThumbValueChangeListener(new MultiSlider.OnThumbValueChangeListener() {
            @Override
            public void onValueChanged(MultiSlider multiSlider, MultiSlider.Thumb thumb, int thumbIndex, int value) {
                if (thumbIndex == 0) {
                    mTextViewMinPrice.setText(Utils.formatPrice(value));
                }
                if (thumbIndex == 1) {
                    if (value < MAX_PRICE) {
                        mTextViewMaxPrice.setText(Utils.formatPrice(value));
                    } else {
                        mTextViewMaxPrice.setText(getResources().getString(R.string.more_than) + Utils.formatPrice(value));
                    }
                }
            }
        });

        return view;
    }

    @Override
    public void onCreateOptionsMenu(Menu menu, MenuInflater inflater) {
        super.onCreateOptionsMenu(menu, inflater);
        inflater.inflate(R.menu.menu_delete, menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.menu_delete:
                initFilter();
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    private void refreshUI() {
        mPriceInfThumb.setValue(mFilter.getPriceInf());
        mPriceSupThumb.setValue(mFilter.getPriceSup());
        mDistanceThumb.setValue(mFilter.getDistSup());
        if (mFilter.getAgrupation() != "0") {
            int agrupationIndex = mAgrupationList.indexOf(new Agrupation(mFilter.getAgrupation(), ""));
            mAgrupationsSpinner.setSelection(agrupationIndex);
        } else {
            mAgrupationsSpinner.setSelection(0);
        }
        mTextViewMinPrice.setText(String.valueOf(Utils.formatPrice(mFilter.getPriceInf())));
        mTextViewMaxPrice.setText(getResources().getString(R.string.more_than) + String.valueOf(Utils.formatPrice(mFilter.getPriceSup())));
        mTextViewMaxDistance.setText(getResources().getString(R.string.more_than) + String.valueOf(Utils.formatDistance(mFilter.getDistSup())));
        mTextViewSearch.setText(mFilter.getDescription());
    }

    private void initFilter() {
        mFilter = new Filter(MAX_PRICE,  MAX_DISTANCE);
        refreshUI();
    }

    @OnClick(R.id.filter_apply)
    public void applyFilter() {

        mFilter.setPriceSup(mPriceSupThumb.getValue());

        mFilter.setPriceInf(mPriceInfThumb.getValue());

        if (mTextViewSearch.getText() != "") {
            mFilter.setDescription(mTextViewSearch.getText().toString());
        }
        mFilter.setAgrupation(((Agrupation) mAgrupationsSpinner.getSelectedItem()).getId());

        if (mDistanceThumb.getValue() < MAX_DISTANCE) {
            if (mLocationPermissionGranted) {
                mFusedLocationProviderClient = LocationServices.getFusedLocationProviderClient(getContext());
                getDeviceLocation();
            } else {
                getLocationPermission();
            }
        } else {
            mFilter.setDistSup(MAX_DISTANCE);
            saveFilter();
            sendDataToListadoAnuncios();
        }
    }

    private void sendDataToListadoAnuncios() {

        Intent i = new Intent();
        i.putExtra(EXTRA_FILTER, mFilter);
        getActivity().setResult(Activity.RESULT_OK, i);
        getActivity().finish();
    }

    private void getLocationPermission() {
        if (ContextCompat.checkSelfPermission(getContext(),
                android.Manifest.permission.ACCESS_FINE_LOCATION)
                == PackageManager.PERMISSION_GRANTED) {
            mLocationPermissionGranted = true;
            getDeviceLocation();
        } else {
            if (shouldShowRequestPermissionRationale(android.Manifest.permission.ACCESS_FINE_LOCATION)) {
                showMessageOKCancel(getResources().getString(R.string.informar_direccion_permission),
                        new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                                requestPermissions(new String[]{android.Manifest.permission.ACCESS_FINE_LOCATION}, REQUEST_ACCESS_FINE_LOCATION);
                            }
                        });
                return;
            }
            else {
                requestPermissions(new String[]{android.Manifest.permission.ACCESS_FINE_LOCATION}, REQUEST_ACCESS_FINE_LOCATION);
                return;
            }
        }
    }

    private void showMessageOKCancel(String message, DialogInterface.OnClickListener okListener) {
        new AlertDialog.Builder(getActivity())
                .setMessage(message)
                .setPositiveButton("OK", okListener)
                .setNegativeButton("Cancel", null)
                .create()
                .show();
    }

    private void getDeviceLocation() {
/*
     * Get the best and most recent location of the device, which may be null in rare
     * cases when a location is not available.
     */
        try {
            if (mLocationPermissionGranted) {
                @SuppressLint("MissingPermission") Task locationResult = mFusedLocationProviderClient.getLastLocation();
                locationResult.addOnCompleteListener(getActivity(), new OnCompleteListener() {
                    @Override
                    public void onComplete(@NonNull Task task) {
                        if (task.isSuccessful() && task.getResult() != null) {
                            // Set the map's camera position to the current location of the device.
                            mLastKnownLocation = (Location) task.getResult();
                            mFilter.setPosition(mLastKnownLocation);
                            mFilter.setDistSup(mDistanceThumb.getValue());
                            saveFilter();
                            sendDataToListadoAnuncios();

                        } else if (task.isSuccessful() && task.getResult() == null) {
                            getCurrentLocation();
                        } else {
                            Log.d(TAG, "Current location is null. Using defaults.");
                            Log.e(TAG, "Exception: %s", task.getException());
                        }
                    }
                });
            }
        } catch (SecurityException e) {
            Log.e("Exception: %s", e.getMessage());
        }
    }

    @SuppressWarnings("MissingPermission")
    private void getCurrentLocation() {
        @SuppressLint("RestrictedApi") LocationRequest locationRequest = new LocationRequest();
        locationRequest.setPriority(LocationRequest.PRIORITY_BALANCED_POWER_ACCURACY)
                .setNumUpdates(1)
                .setExpirationDuration(10*1000);
        mHandler.postDelayed(mExpired, 10*1000);
        mProgressBar.setVisibility(View.VISIBLE);
        getActivity().getWindow().setFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE, WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);
        mFusedLocationProviderClient.requestLocationUpdates(locationRequest, new LocationCallback() {
                    @Override
                    public void onLocationResult(LocationResult locationResult) {
                        // do work here
                        mHandler.removeCallbacks(mExpired);
                        mProgressBar.setVisibility(View.GONE);
                        getActivity().getWindow().clearFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);
                        mLastKnownLocation = locationResult.getLastLocation();
                        mFilter.setPosition(mLastKnownLocation);
                        mFilter.setDistSup(mDistanceThumb.getValue());
                        saveFilter();
                        sendDataToListadoAnuncios();
                    }
                },
                Looper.myLooper());
    }

    private final Runnable mExpired = new Runnable() {
        @Override
        public void run() {
            Utils.alertDialogLogged(getActivity(),getActivity().getResources().getString(R.string.filter_no_position));
            mProgressBar.setVisibility(View.GONE);
            getActivity().getWindow().clearFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);
        }
    };

    @Override
    public void onPause() {
        super.onPause();
        mProgressBar.setVisibility(View.GONE);
        getActivity().getWindow().clearFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);
        mHandler.removeCallbacks(mExpired);
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) {
        mLocationPermissionGranted = false;
        switch (requestCode) {
            case REQUEST_ACCESS_FINE_LOCATION:
                if (grantResults.length > 0
                        && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    mLocationPermissionGranted = true;
                    getDeviceLocation();
                }
                break;
            default:
                super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        }
    }

    private void saveFilter() {
        GsonBuilder builder = new GsonBuilder();
        Gson gson = builder.create();
        String jsonFilter = gson.toJson(mFilter);

        SharedPreferences preferences = getActivity().getSharedPreferences(PREFERENCES_FILTER, Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = preferences.edit();

        Filter initFilter= newFilter();
        if(mFilter.equals(initFilter)){
            editor.clear();
        }
        else {
            editor.putString(PREFERENCES_KEY, jsonFilter);
        }
        editor.commit();

    }
}
