package com.example.xavierferraronsserra.sobreruedas;

import android.Manifest;
import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.location.Address;
import android.location.Geocoder;
import android.location.Location;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AlertDialog;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.ProgressBar;

import com.example.xavierferraronsserra.sobreruedas.Utils.Utils;
import com.google.android.gms.common.api.Status;
import com.google.android.gms.location.FusedLocationProviderClient;
import com.google.android.gms.location.LocationCallback;
import com.google.android.gms.location.LocationRequest;
import com.google.android.gms.location.LocationResult;
import com.google.android.gms.location.LocationServices;
import com.google.android.gms.location.places.Place;
import com.google.android.gms.location.places.ui.PlaceSelectionListener;
import com.google.android.gms.location.places.ui.SupportPlaceAutocompleteFragment;
import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.OnMapReadyCallback;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;

import java.io.IOException;
import java.util.List;
import java.util.Locale;

import butterknife.BindView;
import butterknife.ButterKnife;

/**
 * Created by xavierferraronsserra on 31/3/18.
 */

public class InformarDireccionFragment extends Fragment implements OnMapReadyCallback, GoogleMap.OnMyLocationButtonClickListener {

    private static final String TAG = "InformarDireccionFrag";
    //Argument that contains user location if exists in his profile
    private static final String ARG_LOCATION = "user_location";
    //When user returns to his profile we pass location and address data
    public static final String EXTRA_LOCATION = "com.example.xavierferraronsserra.sobreruedas.location";
    public static final String EXTRA_ADDRESS = "com.example.xavierferraronsserra.sobreruedas.address";
    //Id that identifies the fine location request
    private final int REQUEST_ACCESS_FINE_LOCATION = 1;

    private SupportMapFragment mMapFragment;
    private SupportPlaceAutocompleteFragment mAutocompleteFragment;
    private GoogleMap mMap;
    private final int DEFAULT_ZOOM = 14;

    private FusedLocationProviderClient mFusedLocationProviderClient;
    private Location mAddressLocation;
    private Location mLastKnownLocation;
    private Handler mHandler;

    @BindView(R.id.informar_direccion_progress_bar)
    ProgressBar mProgressBar;


    //Indicates if user has location permissions granted
    private boolean mLocationPermissionGranted = false;

    public static InformarDireccionFragment newInstance(double[] currentLocation) {
        Bundle args = new Bundle();
        args.putDoubleArray(ARG_LOCATION, currentLocation);
        InformarDireccionFragment informarDireccionFragment = new InformarDireccionFragment();
        informarDireccionFragment.setArguments(args);
        return informarDireccionFragment;
    }

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setHasOptionsMenu(true);
        double[] location = null;
        mHandler=new Handler();

        //We check if user has defined location data in his profile
        location = getArguments().getDoubleArray(ARG_LOCATION);
        if (location != null) {
            mAddressLocation = new Location("");
            mAddressLocation.setLatitude(location[0]);
            mAddressLocation.setLongitude(location[1]);
        }
        mFusedLocationProviderClient = LocationServices.getFusedLocationProviderClient(getContext());
    }

    @Nullable
    @Override
    public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_informar_direccion, container, false);
        ButterKnife.bind(this,view);
        return view;
    }

    @Override
    public void onActivityCreated(@Nullable Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);
        mMapFragment = (SupportMapFragment) getChildFragmentManager().findFragmentById(R.id.informar_direccion_map);
        mAutocompleteFragment = (SupportPlaceAutocompleteFragment) getChildFragmentManager().findFragmentById(R.id.informar_direccion_place_autocomplete_fragment);
        mAutocompleteFragment.setOnPlaceSelectedListener(new PlaceSelectionListener() {
            @Override
            public void onPlaceSelected(Place place) {
                Geocoder gc = new Geocoder(getContext());
                List<Address> list = null;
                try {
                    list = gc.getFromLocationName(place.getName().toString(), 1);
                } catch (IOException e) {
                    e.printStackTrace();
                }
                Address address = list.get(0);
                mAddressLocation = new Location("");
                mAddressLocation.setLatitude(address.getLatitude());
                mAddressLocation.setLongitude(address.getLongitude());
                addMarker(new LatLng(mAddressLocation.getLatitude(),mAddressLocation.getLongitude()));

                Log.i(TAG, "Place: " + place.getName());
            }

            @Override
            public void onError(Status status) {
                // TODO: Handle the error.
                Log.i(TAG, "An error occurred: " + status);
            }
        });

        mMapFragment.getMapAsync(this);
    }

    @Override
    public void onCreateOptionsMenu(Menu menu, MenuInflater inflater) {
        super.onCreateOptionsMenu(menu, inflater);
        inflater.inflate(R.menu.menu_save, menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {

        if (item.getItemId() == R.id.menu_save) {
            sendLocation();
            return true;
        } else {
            getActivity().finish();
            return true;
        }
    }

    @Override
    public void onMapReady(GoogleMap googleMap) {
        mMap = googleMap;
        mMap.setOnMyLocationButtonClickListener(this);

        //Asks for location permissions
        getLocationPermission();
        //We update UI map controls
        updateUI();
        if(mAddressLocation!=null){
            addMarker(new LatLng(mAddressLocation.getLatitude(),mAddressLocation.getLongitude()));
        }

    }

    private void updateUI() {
        updateLocationUI();
        updateZoomUI();
    }

    private void updateZoomUI() {
        if (mMap == null) {
            return;
        }
        mMap.getUiSettings().setZoomControlsEnabled(true);
    }

    private void updateLocationUI() {
        if (mMap == null) {
            return;
        }
        try {
            if (mLocationPermissionGranted) {
                mMap.setMyLocationEnabled(true);
                mMap.getUiSettings().setMyLocationButtonEnabled(true);
            } else {
                mMap.setMyLocationEnabled(false);
                mMap.getUiSettings().setMyLocationButtonEnabled(false);
                mLastKnownLocation = null;
            }
        } catch (SecurityException e) {
            Log.e("Exception: %s", e.getMessage());
        }
    }

    private void getDeviceLocation() {
/*
     * Get the best and most recent location of the device, which may be null in rare
     * cases when a location is not available.
     */
        mProgressBar.setVisibility(View.VISIBLE);
        getActivity().getWindow().setFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE, WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);

        try {
            if (mLocationPermissionGranted) {
                @SuppressLint("MissingPermission") Task locationResult = mFusedLocationProviderClient.getLastLocation();
                locationResult.addOnCompleteListener(getActivity(), new OnCompleteListener() {
                    @Override
                    public void onComplete(@NonNull Task task) {
                        if (task.isSuccessful() && task.getResult() != null) {
                            Log.d(TAG, "Last known location is not null.");
                            // if we can get last known location set it as last known location
                            mProgressBar.setVisibility(View.GONE);
                            getActivity().getWindow().clearFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);
                            mLastKnownLocation = (Location) task.getResult();
                            if(mAddressLocation==null) {
                                mAddressLocation=mLastKnownLocation;
                                addMarker(new LatLng(mAddressLocation.getLatitude(), mAddressLocation.getLongitude()));
                            }
                        } else {
                            Log.d(TAG, "Last known location is null getting current location");
                            getCurrentLocation();
                        }
                    }
                });
            }
        } catch (SecurityException e) {
            Log.e("Exception: %s", e.getMessage());
        }
    }

    @SuppressWarnings("MissingPermission")
    private void getCurrentLocation() {
        @SuppressLint("RestrictedApi") LocationRequest locationRequest = new LocationRequest();
        locationRequest.setPriority(LocationRequest.PRIORITY_BALANCED_POWER_ACCURACY)
                .setNumUpdates(1)
                .setExpirationDuration(10*1000);
        mHandler.postDelayed(mExpired, 10*1000);
        mFusedLocationProviderClient.requestLocationUpdates(locationRequest, new LocationCallback() {
                    @Override
                    public void onLocationResult(LocationResult locationResult) {
                        mProgressBar.setVisibility(View.GONE);
                        getActivity().getWindow().clearFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);
                        mLastKnownLocation = locationResult.getLastLocation();
                        if(mAddressLocation==null) {
                            mAddressLocation=mLastKnownLocation;
                            addMarker(new LatLng(mAddressLocation.getLatitude(), mAddressLocation.getLongitude()));
                        }
                    }
                },
                Looper.myLooper());
    }

    private final Runnable mExpired = new Runnable() {
        @Override
        public void run() {
            mProgressBar.setVisibility(View.GONE);
            getActivity().getWindow().clearFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);
            Utils.alertDialogLogged(getActivity(),getActivity().getResources().getString(R.string.validation_no_position));
            if(mAddressLocation!=null) {
                addMarker(new LatLng(mAddressLocation.getLatitude(), mAddressLocation.getLongitude()));
            }
        }
    };

    @Override
    public void onPause() {
        super.onPause();
        mHandler.removeCallbacks(mExpired);
    }


    private void addMarker(LatLng position) {
        mMap.clear();
        MarkerOptions markerOptions = new MarkerOptions();
        markerOptions.position(position);
        mMap.addMarker(markerOptions);
        mMap.moveCamera(CameraUpdateFactory.newLatLngZoom(
                position, DEFAULT_ZOOM));
    }

    private void showMessageOKCancel(String message, DialogInterface.OnClickListener okListener) {
        new AlertDialog.Builder(getActivity())
                .setMessage(message)
                .setPositiveButton("OK", okListener)
                .setNegativeButton("Cancel", null)
                .create()
                .show();
    }

    private void getLocationPermission() {
        if (ContextCompat.checkSelfPermission(getContext(),
    android.Manifest.permission.ACCESS_FINE_LOCATION)
            == PackageManager.PERMISSION_GRANTED) {
        mLocationPermissionGranted = true;
        getDeviceLocation();
    } else {
        if (shouldShowRequestPermissionRationale(Manifest.permission.ACCESS_FINE_LOCATION)) {
            showMessageOKCancel(getResources().getString(R.string.informar_direccion_permission),
                    new DialogInterface.OnClickListener() {
                        @Override
                        public void onClick(DialogInterface dialog, int which) {
                            requestPermissions(new String[]{Manifest.permission.ACCESS_FINE_LOCATION}, REQUEST_ACCESS_FINE_LOCATION);
                        }
                    });
            return;
        }
        else {
            requestPermissions(new String[]{Manifest.permission.ACCESS_FINE_LOCATION}, REQUEST_ACCESS_FINE_LOCATION);
            return;
        }
    }
}

    @Override
    public void onRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) {
        mLocationPermissionGranted = false;
        switch (requestCode) {
            case REQUEST_ACCESS_FINE_LOCATION:
                // If request is cancelled, the result arrays are empty.
                if (grantResults.length > 0
                        && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    mLocationPermissionGranted = true;
                    updateLocationUI();
                    getDeviceLocation();
                }
                break;
            default:
                super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        }
    }

    private void sendLocation() {
        Intent i = new Intent();
        if (mAddressLocation != null) {
            i.putExtra(EXTRA_LOCATION, mAddressLocation);
            String address = getAddressFromLocation(mAddressLocation.getLatitude(), mAddressLocation.getLongitude());
            i.putExtra(EXTRA_ADDRESS, address);
            getActivity().setResult(Activity.RESULT_OK, i);
            getActivity().finish();
        } else {
            getActivity().setResult(Activity.RESULT_CANCELED);
            getActivity().finish();
        }
    }

    private String getAddressFromLocation(double latitude, double longitude) {

        Geocoder geocoder = new Geocoder(getContext(), Locale.getDefault());
        try {
            List<Address> addresses = geocoder.getFromLocation(latitude, longitude, 1);
            if (addresses.size() > 0) {
                return addresses.get(0).getLocality() + " (" + addresses.get(0).getSubAdminArea() + ")";
            }
        } catch (IOException e) {
            e.printStackTrace();
            Log.e(TAG, e.toString());
        }
        return "";
    }

    @Override
    public boolean onMyLocationButtonClick() {
        mAddressLocation=null;
        getDeviceLocation();
        return true;
    }
}
