package com.example.xavierferraronsserra.sobreruedas;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.design.widget.NavigationView;
import android.support.multidex.MultiDex;
import android.support.v4.app.Fragment;
import android.support.v4.widget.DrawerLayout;
import android.support.v7.app.ActionBarDrawerToggle;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.MenuItem;
import android.view.View;
import android.widget.TextView;

import com.example.xavierferraronsserra.sobreruedas.Model.Usuario;
import com.example.xavierferraronsserra.sobreruedas.Utils.StorageManagement;
import com.example.xavierferraronsserra.sobreruedas.Utils.Utils;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.firestore.DocumentSnapshot;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;

import static com.example.xavierferraronsserra.sobreruedas.R.id.navheader_email;

/**
 * Created by xavierferraronsserra on 29/3/18.
 */

public class ListadoAnunciosActivity extends SingleFragmentActivity implements NavigationView.OnNavigationItemSelectedListener {

    private static final String TAG = "ListadoAnunciosActivity";
    private FirebaseAuth mAuth;
    private Usuario mUsuario;


    @BindView(R.id.navheader_login)  TextView loginTextView;
    @BindView(R.id.navheader_register)  TextView registerTextView;
    @BindView(R.id.navheader_where)  TextView whereTextView;
    @BindView(R.id.navheader_name)  TextView nameTextView;
    @BindView(navheader_email) TextView emailTextView;

    @Override
    protected void attachBaseContext(Context base) {
        super.attachBaseContext(base);
        MultiDex.install(this);
    }

    @Override
    protected Fragment createFragment() {
        return new ListadoAnunciosFragment();
    }

    @Override
    protected int getLayoutResId() {
        return R.layout.activity_menu_fragment;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {

        super.onCreate(savedInstanceState);

        mAuth= FirebaseAuth.getInstance();

        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
        ActionBarDrawerToggle toggle = new ActionBarDrawerToggle(
                this, drawer, toolbar, R.string.navigation_drawer_open, R.string.navigation_drawer_close);
        drawer.addDrawerListener(toggle);
        toggle.syncState();

        NavigationView navigationView = (NavigationView) findViewById(R.id.nav_view);
        navigationView.setNavigationItemSelectedListener(this);

        View headerView = navigationView.getHeaderView(0);

        ButterKnife.bind(this, headerView);

        if (mAuth.getCurrentUser() != null) {
            nameTextView.setVisibility(View.VISIBLE);
            emailTextView.setVisibility(View.VISIBLE);
            registerTextView.setVisibility(View.GONE);
            whereTextView.setVisibility(View.GONE);
            loginTextView.setVisibility(View.GONE);

            StorageManagement.getUser(getApplicationContext(), mAuth.getCurrentUser().getUid()).get().addOnCompleteListener(new OnCompleteListener<DocumentSnapshot>() {
                @Override
                public void onComplete(@NonNull Task<DocumentSnapshot> task) {
                    if (task.isSuccessful()) {
                        mUsuario =task.getResult().toObject(Usuario.class);
                        nameTextView.setText(mUsuario.getNombre()+ " " + mUsuario.getApellidos());
                        emailTextView.setText(mUsuario.getEmail());

                        if (mUsuario != null) {
                            Log.d(TAG, "DocumentSnapshot data: " + task.getResult().getData());
                        } else {
                            Log.w(TAG, "No such document");
                        }
                    } else {
                        Log.w(TAG, "get failed with ", task.getException());
                    }
                }
            });
        } else {
            nameTextView.setVisibility(View.GONE);
            emailTextView.setVisibility(View.GONE);
            registerTextView.setVisibility(View.VISIBLE);
            whereTextView.setVisibility(View.VISIBLE);
            loginTextView.setVisibility(View.VISIBLE);
        }
    }

    @OnClick()
    public void submit(View view) {
        if(mAuth.getCurrentUser()!=null){
            Intent updateUsuario = ModificarUsuarioActivity.newIntent(getApplicationContext(),mUsuario);
            startActivity(updateUsuario);
        }
        else {
            Intent i = new Intent(this, LoginActivity.class);
            startActivity(i);
        }
    }


    @SuppressWarnings("StatementWithEmptyBody")
    @Override
    public boolean onNavigationItemSelected(MenuItem item) {
        // Handle navigation view item clicks here.
        switch (item.getItemId()){
            case R.id.nav_disconnect:
                mAuth.signOut();
                Intent signOutIntent = new Intent(this, ListadoAnunciosActivity.class);
                signOutIntent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
                startActivity(signOutIntent);
                break;
            case R.id.nav_my_advetisements:
                if(mAuth.getCurrentUser()!=null) {
                    Intent misAnunciosIntent = new Intent(this, ListadoAnunciosUsuarioActivity.class);
                    startActivity(misAnunciosIntent);
                }
                else{
                    Utils.alertDialogLogged(this, getResources().getString(R.string.logged_required));
                }
                break;
            case R.id.nav_notifications:
                if(mAuth.getCurrentUser()!=null) {
                    Intent notificacionesIntent = new Intent(this, NotificacionesActivity.class);
                    startActivity(notificacionesIntent);
                }
                else{
                    Utils.alertDialogLogged(this,getResources().getString(R.string.logged_required));
                }
                break;
            case R.id.nav_upload_product:
                if(mAuth.getCurrentUser()!=null){
                    Intent uploadProductIntent = new Intent(this, AltaAnuncioActivity.class);
                    startActivity(uploadProductIntent);
                }
                else{
                    Utils.alertDialogLogged(this,getResources().getString(R.string.logged_required));
                }
                break;
        }



        return true;
    }



}
