package com.example.xavierferraronsserra.sobreruedas;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.support.design.widget.FloatingActionButton;
import android.support.v4.app.ActivityCompat;
import android.support.v4.app.Fragment;
import android.support.v4.content.ContextCompat;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.ProgressBar;

import com.algolia.search.saas.AlgoliaException;
import com.algolia.search.saas.CompletionHandler;
import com.algolia.search.saas.Index;
import com.example.xavierferraronsserra.sobreruedas.Adapters.AnunciosAdapter;
import com.example.xavierferraronsserra.sobreruedas.Model.Anuncio;
import com.example.xavierferraronsserra.sobreruedas.Model.Filter;
import com.example.xavierferraronsserra.sobreruedas.Utils.StorageManagement;
import com.example.xavierferraronsserra.sobreruedas.Utils.Utils;
import com.google.firebase.auth.FirebaseAuth;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;

import static com.example.xavierferraronsserra.sobreruedas.FilterFragment.PREFERENCES_FILTER;
import static com.example.xavierferraronsserra.sobreruedas.FilterFragment.PREFERENCES_KEY;

/**
 * Created by xavierferraronsserra on 29/3/18.
 */

public class ListadoAnunciosFragment extends Fragment implements AnunciosAdapter.AnunciosHolder.ClickListener {

    private final String TAG = "ListadoAnunciosFragment";

    private static final int REQUEST_FILTER = 1;
    private static final int REQUEST_PRODUCT = 2;

    private FirebaseAuth mAuth;
    private Index mIndex;

    private ArrayList<Anuncio> mAnuncios = new ArrayList<Anuncio>();
    //Indicates number of new items added to mAnuncios since last algolia data loading
    private int mInsertedItems;
    private AnunciosAdapter mAdapter;
    private LinearLayoutManager mly;
    private EndlessRecyclerViewScrollListener mScrollListener;
    private Filter mFilter;

    @BindView(R.id.listado_anuncios_fab)
    FloatingActionButton mFloatingActionButton;
    @BindView(R.id.anuncios_recycler_view)
    RecyclerView mRecyclerView;
    @BindView(R.id.listado_anuncios_progress_bar)
    ProgressBar mProgressBar;


    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setHasOptionsMenu(true);
        mFilter = FilterFragment.newFilter();
        mAuth = FirebaseAuth.getInstance();
        //Checks if exists some filter saved in preferences
        SharedPreferences preferences = getActivity().getSharedPreferences(PREFERENCES_FILTER, Context.MODE_PRIVATE);
        if (preferences.contains(PREFERENCES_KEY)) {
            GsonBuilder builder = new GsonBuilder();
            Gson gson = builder.create();
            mFilter = gson.fromJson(preferences.getString(PREFERENCES_KEY, ""), Filter.class);
        }
    }

    @Nullable
    @Override
    public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {

        View view = inflater.inflate(R.layout.fragment_listado_anuncios, container, false);
        ButterKnife.bind(this, view);

        mProgressBar.setVisibility(View.VISIBLE);
        getActivity().getWindow().clearFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);


        initRecyclerView();
        //Get first pagination data
        getAnuncios(0, mFilter);
        return view;
    }

    @Override
    public void onCreateOptionsMenu(Menu menu, MenuInflater inflater) {
        super.onCreateOptionsMenu(menu, inflater);
        inflater.inflate(R.menu.menu_listado_anuncios, menu);
    }

    @Override
    public void onPrepareOptionsMenu(Menu menu) {
        super.onPrepareOptionsMenu(menu);
        SharedPreferences preferences = getActivity().getSharedPreferences(PREFERENCES_FILTER, Context.MODE_PRIVATE);
        //Checks if exists some defined filter in order to indicate it in the menu with the icon color
        if (preferences.contains(PREFERENCES_KEY)) {
            menu.getItem(0).setIcon(ContextCompat.getDrawable(getContext(), R.drawable.ic_filter_red));
        } else {
            menu.getItem(0).setIcon(ContextCompat.getDrawable(getContext(), R.drawable.ic_filter));
        }
    }

    @Override
    public void onResume() {
        super.onResume();
        //Invalidate menu in order to check if there is a defined filter
        ActivityCompat.invalidateOptionsMenu(getActivity());
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.menu_listado_anuncios_search:
                Intent filterIntent = new Intent(getContext(), FilterActivity.class);
                startActivityForResult(filterIntent, REQUEST_FILTER);
                return true;
            case R.id.menu_listado_anuncios_map:
                Intent mapaIntent = new Intent(getContext(), ListadoAnunciosMapaActivity.class);
                startActivity(mapaIntent);
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    @OnClick(R.id.listado_anuncios_fab)
    public void goToNuevoAnuncio() {
        if (mAuth.getCurrentUser() != null) {
            Intent i = new Intent(getContext(), AltaAnuncioActivity.class);
            startActivityForResult(i, REQUEST_PRODUCT);
        } else {
            Utils.alertDialogLogged(getActivity(), getActivity().getResources().getString(R.string.logged_required));
        }
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (resultCode != Activity.RESULT_OK) {
            return;
        }
        if (requestCode == REQUEST_FILTER) {
            Filter filter = (Filter) data.getParcelableExtra(FilterFragment.EXTRA_FILTER);
            //Resets data
            mAnuncios.clear();
            //Gets first pagination data
            getAnuncios(0, filter);
            //Informs to the adapter that new data has been sent
            mAdapter.notifyDataSetChanged();
        }
        if (requestCode == REQUEST_PRODUCT) {
            //Gets new added product
            Anuncio anuncio = (Anuncio) data.getParcelableExtra(AltaAnuncioFragment.EXTRA_ANUNCIO);
            //Insert new product as first product in the list
            mAnuncios.add(0, anuncio);
            //We control how many products have been added since last algolia data load
            mInsertedItems++;
            //Informs to the adapter that a new item has been inserted
            mAdapter.notifyItemInserted(0);
            //Scrolls list to the first position
            mRecyclerView.scrollToPosition(0);
        }
    }

    private void initRecyclerView() {
        mAdapter = new AnunciosAdapter(getContext(), mAnuncios, (AnunciosAdapter.AnunciosHolder.ClickListener) this);
        mly = new LinearLayoutManager(getActivity());
        mRecyclerView.setLayoutManager(mly);
        mRecyclerView.setAdapter(mAdapter);
        //Scroll listener that detects when we have to load new data
        mScrollListener = new EndlessRecyclerViewScrollListener(mly) {
            @Override
            public void onLoadMore(int page, int totalItemsCount, RecyclerView view) {
                getAnuncios(page, mFilter);
            }
        };
        mRecyclerView.addOnScrollListener(mScrollListener);
    }

    private void getAnuncios(final int page, Filter filter) {
        //Sets the query to algolia depending on the filter params
        final com.algolia.search.saas.Query query = StorageManagement.getAnunciosAlgolia(getContext(), filter);
        //Informs de page to request
        query.setPage(page);
        mIndex = StorageManagement.getIndex();
        //In the first request we set the index to inform the number of items that have every pagination request
        if (page == 0) {
            try {
                mIndex.setSettingsAsync(new JSONObject().put("hitsPerPage", StorageManagement.getPageItems()), new CompletionHandler() {
                    @Override
                    public void requestCompleted(JSONObject jsonObject, AlgoliaException e) {
                        Log.w(TAG, "Algolia request completed");
                        //Request algolia query and page data
                        getAlgoliaQueryData(query, page);
                    }
                });
            } catch (JSONException e) {
                e.printStackTrace();
            }
        } else {
            //After the first request we have the index configured so we can request data directly
            getAlgoliaQueryData(query, page);
        }
    }

    public void getAlgoliaQueryData(com.algolia.search.saas.Query query, final int page) {
        mIndex.searchAsync(query, new CompletionHandler() {
            @Override
            public void requestCompleted(JSONObject jsonObject, AlgoliaException e) {
                Log.w(TAG, "Algolia request completed");
                List<Anuncio> anuncios = Utils.jsonToObjectAnuncios(jsonObject);
                mAnuncios.addAll(anuncios);
                if (page == 0) {
                    mProgressBar.setVisibility(View.GONE);
                    getActivity().getWindow().clearFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);
                    if (anuncios.isEmpty()) {
                        alertNoData(getActivity());
                    }
                }
                //informs to the adapter the range of new added items
            mAdapter.notifyDataSetChanged();
            }
        });
    }

    @Override
    public void onItemClicked(Anuncio anuncio) {
        Intent detalleAnuncio = DetalleAnuncioActivity.newIntent(getContext(), anuncio);
        startActivity(detalleAnuncio);
    }

    public AlertDialog.Builder alertNoData(Context ctx) {
        AlertDialog.Builder builder = new AlertDialog.Builder(ctx);
        builder.setMessage(getResources().getString(R.string.no_data_alert));
        builder.setPositiveButton("OK", null);
        builder.show();
        return builder;
    }
}


