package com.example.xavierferraronsserra.sobreruedas;

import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ProgressBar;

import com.algolia.search.saas.AlgoliaException;
import com.algolia.search.saas.CompletionHandler;
import com.algolia.search.saas.Index;
import com.example.xavierferraronsserra.sobreruedas.Model.Anuncio;
import com.example.xavierferraronsserra.sobreruedas.Model.Filter;
import com.example.xavierferraronsserra.sobreruedas.Utils.StorageManagement;
import com.example.xavierferraronsserra.sobreruedas.Utils.Utils;
import com.google.android.gms.maps.CameraUpdate;
import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.OnMapReadyCallback;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.LatLngBounds;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;

import static com.example.xavierferraronsserra.sobreruedas.FilterFragment.PREFERENCES_FILTER;
import static com.example.xavierferraronsserra.sobreruedas.FilterFragment.PREFERENCES_KEY;

/**
 * Created by xavierferraronsserra on 31/3/18.
 */

public class ListadoAnunciosMapaFragment extends Fragment implements OnMapReadyCallback, GoogleMap.OnInfoWindowClickListener {

    private static final String TAG = "ListadoAnunciosMFrag";
    private static final String ARG_ANUNCIOS = "listado_anuncios";
    private SupportMapFragment mMapFragment;

    private GoogleMap mMap;
    private ArrayList<Marker> mMarkers = new ArrayList<>();
    private LatLngBounds.Builder mBuilder = new LatLngBounds.Builder();
    private final int DEFAULT_ZOOM = 14;

    private List<Anuncio> mAnuncios;
    private Filter mFilter;

    @BindView(R.id.listado_anuncios_mapa_progress_bar)
    ProgressBar mProgressBar;

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setHasOptionsMenu(true);
        //Checks if exists a defined filter
        SharedPreferences preferences = getActivity().getSharedPreferences(PREFERENCES_FILTER, Context.MODE_PRIVATE);
        if (preferences.contains(PREFERENCES_KEY)) {
            GsonBuilder builder = new GsonBuilder();
            Gson gson = builder.create();
            mFilter = gson.fromJson(preferences.getString(PREFERENCES_KEY, ""), Filter.class);
        }
        //If not exists any filter we create a filter without filter options
        else {
            mFilter = FilterFragment.newFilter();
        }
    }

    @Nullable
    @Override
    public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {

        View view = inflater.inflate(R.layout.fragment_listado_anuncios_mapa, container, false);
        ButterKnife.bind(this, view);
        mProgressBar.setVisibility(View.VISIBLE);

        return view;
    }

    @Override
    public void onActivityCreated(@Nullable Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);
        mMapFragment = (SupportMapFragment) getChildFragmentManager().findFragmentById(R.id.listado_anuncios_map);
        mMapFragment.getMapAsync(this);

    }

    @Override
    public void onMapReady(GoogleMap googleMap) {
        mMap = googleMap;
        mMap.setOnInfoWindowClickListener(this);
        updateUI();
        final com.algolia.search.saas.Query query = StorageManagement.getAnunciosAlgolia(getContext(), mFilter);
        final Index index = StorageManagement.getIndex();
        //We obtain data amb add markers to the map
        try {
            index.setSettingsAsync(new JSONObject().put("hitsPerPage", StorageManagement.getMaxItems()), new CompletionHandler() {
                @Override
                public void requestCompleted(JSONObject jsonObject, AlgoliaException e) {
                    Log.w(TAG, "Algolia request completed");
                    index.searchAsync(query, new CompletionHandler() {
                        @Override
                        public void requestCompleted(JSONObject jsonObject, AlgoliaException e) {
                            Log.w(TAG, "Algolia request completed");
                            mAnuncios = Utils.jsonToObjectAnuncios(jsonObject);
                            addMarkers();
                            mProgressBar.setVisibility(View.GONE);
                        }
                    });
                }
            });
        } catch (JSONException e) {
            e.printStackTrace();
        }
    }

    private void updateUI() {
        updateZoomUI();
    }

    private void updateZoomUI() {
        if (mMap == null) {
            return;
        }
        mMap.getUiSettings().setZoomControlsEnabled(true);
    }

    private void addMarkers() {
        //We clean previous markers
        mMap.clear();
        MarkerOptions markerOptions = new MarkerOptions();

        //We create markers with data
        for (int i = 0; i < mAnuncios.size(); i++) {
            Anuncio anuncio = mAnuncios.get(i);
            LatLng positionLatLng = new LatLng(anuncio.getPosicion().getLatitude(), anuncio.getPosicion().getLongitude());
            Marker marker = mMap.addMarker(markerOptions
                    .position(positionLatLng)
                    .title(anuncio.getTitle() + " (" + Utils.formatPrice(anuncio.getPrice()).toString() + ")")
            );
            marker.setTag(anuncio);
            mMarkers.add(marker);
            mBuilder.include(marker.getPosition());
        }

        LatLngBounds bounds = mBuilder.build();
        int padding = 100; // offset from edges of the map in pixels
        CameraUpdate cu = CameraUpdateFactory.newLatLngBounds(bounds, padding);
        mMap.animateCamera(cu);
    }

    @Override
    public void onInfoWindowClick(Marker marker) {
        Intent detalleAnuncio = DetalleAnuncioActivity.newIntent(getActivity(), (Anuncio) marker.getTag());
        startActivity(detalleAnuncio);
    }
}
