package com.example.xavierferraronsserra.sobreruedas;

import android.content.Intent;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.ProgressBar;

import com.example.xavierferraronsserra.sobreruedas.Adapters.AnunciosUsuarioAdapter;
import com.example.xavierferraronsserra.sobreruedas.Adapters.FirestoreAdapter;
import com.example.xavierferraronsserra.sobreruedas.Model.Anuncio;
import com.example.xavierferraronsserra.sobreruedas.Utils.StorageManagement;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.firestore.FirebaseFirestore;
import com.google.firebase.firestore.Query;
import com.google.firebase.iid.FirebaseInstanceId;
import com.google.firebase.messaging.FirebaseMessaging;

import org.json.JSONException;

import butterknife.BindView;
import butterknife.ButterKnife;

import static android.view.View.GONE;

/**
 * Created by xavierferraronsserra on 29/3/18.
 */

public class ListadoAnunciosUsuarioFragment extends Fragment implements AnunciosUsuarioAdapter.AnunciosHolder.ClickListener, FirestoreAdapter.FinishLoading {

    @BindView(R.id.anuncios_recycler_view) RecyclerView mRecyclerView;
    @BindView(R.id.listado_anuncios_usuario_progress_bar)
    ProgressBar mProgressBar;

    private final String TAG = "ListadoAnunUsuarFrag";
    private FirebaseFirestore mFirestore;
    private FirebaseMessaging mFCM;
    private FirebaseAuth mAuth;
    private FirebaseInstanceId mInstanceId;
    private Query mQuery;
    private AnunciosUsuarioAdapter mAdapter;
    private LinearLayoutManager mly;
    private android.view.ActionMode mActionMode;



    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setHasOptionsMenu(true);
        mFirestore = FirebaseFirestore.getInstance();
        mAuth = FirebaseAuth.getInstance();
        //StorageManagement.createAgrupations(getContext());
        //StorageManagement.createTopics(getContext());
    }

    @Nullable
    @Override
    public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {

        View view = inflater.inflate(R.layout.fragment_listado_anuncios_usuario,container, false);

        ButterKnife.bind(this, view);
        mProgressBar.setVisibility(View.VISIBLE);
        getActivity().getWindow().clearFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);

        initFirestore();
        initRecyclerView();

        return view;
    }


    private void initFirestore(){
        mQuery = StorageManagement.getAnunciosUsuario(getContext(),mAuth.getCurrentUser().getUid().toString());
    }

    private void initRecyclerView(){
        if (mQuery == null) {
            Log.w(TAG, "No query, not initializing RecyclerView");
        }
        mAdapter = new AnunciosUsuarioAdapter(getContext(),mQuery, (AnunciosUsuarioAdapter.AnunciosHolder.ClickListener)this, (FirestoreAdapter.FinishLoading)this);
        mly =new LinearLayoutManager(getActivity());
        mRecyclerView.setLayoutManager(mly);
        mRecyclerView.setAdapter(mAdapter);
    }

    @Override
    public void onItemClicked(int position, Anuncio anuncio) {
        if(mAdapter.isMultiselection()==false) {
            Intent modificarAnuncio = AltaAnuncioUsuarioActivity.newIntent(getContext(), anuncio);
            startActivity(modificarAnuncio);
        }
    }

    @Override
    public boolean onItemLongClicked(int position) {
        toogleSelection(position);
        if(mAdapter.isMultiselection()==false){
            mAdapter.setMultiselection(true);
            mActionMode = getActivity().startActionMode(new ActionBarCallBack());
        }
        return true;
    }

    private void toogleSelection(int position){
        mAdapter.toggleSelection(position);
    }

    @Override
    public boolean finishloading() {
        mProgressBar.setVisibility(GONE);
        getActivity().getWindow().clearFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);

        return true;



    }

    class ActionBarCallBack implements android.view.ActionMode.Callback {

        @Override
        public boolean onCreateActionMode(android.view.ActionMode mode, Menu menu) {
            mode.getMenuInflater().inflate(R.menu.menu_anuncios_usuario, menu);
            return true;
        }

        @Override
        public boolean onPrepareActionMode(android.view.ActionMode mode, Menu menu) {
            return false;
        }

        @Override
        public boolean onActionItemClicked(android.view.ActionMode mode, MenuItem menuItem) {
            switch (menuItem.getItemId()) {
                case R.id.menu_anuncios_usuario_delete:
                    deleteAnuncios();
                    mAdapter.clearSelection();
                    mode.finish();
                    return true;
                case R.id.menu_anuncios_usuario_rennovate:
                    try {
                        changeStateAnuncios();
                    } catch (JSONException e) {
                        e.printStackTrace();
                    }
                    mAdapter.clearSelection();
                    mode.finish();
                    return true;
                default:
                    return false;
            }
        }

        public void deleteAnuncios(){
            try {
                StorageManagement.deleteAnuncios(mAdapter.getSelectedItems());
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        public void changeStateAnuncios() throws JSONException {
            StorageManagement.changeStateAnuncios(mAdapter.getSelectedItems());
        }

        @Override
        public void onDestroyActionMode(android.view.ActionMode mode) {
            mode.finish();
            mAdapter.clearSelection();
        }
    }
    @Override
    public void onStart() {
        super.onStart();
        if (mAdapter != null) {
            mAdapter.startListening();
        }
    }

    @Override
    public void onStop() {
        super.onStop();
        super.onStop();
        if (mAdapter != null) {
            mAdapter.stopListening();
        }
    }


}


