package com.example.xavierferraronsserra.sobreruedas;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.design.widget.TextInputEditText;
import android.support.design.widget.TextInputLayout;
import android.support.v4.app.Fragment;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;

import com.example.xavierferraronsserra.sobreruedas.Utils.StorageManagement;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.auth.AuthResult;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.firestore.DocumentSnapshot;
import com.google.firebase.iid.FirebaseInstanceId;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;

/**
 * Created by xavierferraronsserra on 30/3/18.
 */

public class LoginFragment extends Fragment {

    private static final String TAG = "Login Fragment";
    private static final int REQUEST_USER = 1;
    private FirebaseAuth mAuth;
    private FirebaseInstanceId mInstanceId;
    private boolean mValidate;

    @BindView(R.id.input_login_email)
    TextInputEditText inputEmailEditText;
    @BindView(R.id.input_layout_login_email)
    TextInputLayout inputLayoutEmail;
    @BindView(R.id.input_login_password)
    TextInputEditText inputPasswordEditText;
    @BindView(R.id.input_layout_login_password)
    TextInputLayout inputLayoutPassword;
    @BindView(R.id.text_register_here)
    TextView registerHereTextView;
    @BindView(R.id.login_progress_bar)
    ProgressBar mProgressBar;

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        mAuth = FirebaseAuth.getInstance();
        mInstanceId = FirebaseInstanceId.getInstance();
        mValidate = true;
    }

    @Nullable
    @Override
    public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_login, container, false);
        ButterKnife.bind(this, view);
        setOnFocusChange();
        return view;
    }

    private void setOnFocusChange() {
        inputEmailEditText.setOnFocusChangeListener(new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View v, boolean hasFocus) {
                if (!hasFocus) {
                    inputLayoutEmail.setError(validateEmail(inputEmailEditText.getText().toString()));
                }
            }
        });
        inputPasswordEditText.setOnFocusChangeListener(new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View v, boolean hasFocus) {
                if (!hasFocus) {
                    inputLayoutPassword.setError(validatePassword(inputPasswordEditText.getText().toString()));
                }
            }
        });
    }

    @OnClick(R.id.text_register_here)
    public void register() {
        Intent i = new Intent(getContext(), RegistrarActivity.class);
        startActivityForResult(i, REQUEST_USER);
    }

    @OnClick(R.id.button_login)
    public void login() {
        validate();
        if (mValidate) {
            mProgressBar.setVisibility(View.VISIBLE);
            getActivity().getWindow().setFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE, WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);
            mAuth.signInWithEmailAndPassword(inputEmailEditText.getText().toString(), inputPasswordEditText.getText().toString())
                    .addOnCompleteListener(getActivity(), new OnCompleteListener<AuthResult>() {
                        @Override
                        public void onComplete(@NonNull Task<AuthResult> task) {
                            if (task.isSuccessful()) {
                                Log.d(TAG, "signInWithEmail:success");
                                final String token = mInstanceId.getToken().toString();
                                // Every time user registers we check if token exists.
                                // If not we register new token to the user
                                StorageManagement.getToken(getContext(), mAuth.getCurrentUser().getUid(), token)
                                        .get()
                                        .addOnCompleteListener(new OnCompleteListener<DocumentSnapshot>() {
                                            @Override
                                            public void onComplete(@NonNull Task<DocumentSnapshot> task) {
                                                if (task.isSuccessful() && task.getResult().exists()) {
                                                    Log.d(TAG, "token exists!");
                                                } else {
                                                    StorageManagement.newToken(getContext(), mAuth.getCurrentUser().getUid().toString(), token);
                                                }
                                            }
                                        });
                                sendToListadoAnuncios();
                            } else {
                                // If sign in fails, display a message to the user.
                                Log.w(TAG, "signInWithEmail:failure", task.getException());
                                mProgressBar.setVisibility(View.GONE);
                                getActivity().getWindow().clearFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);
                                Toast.makeText(getContext(), getResources().getString(R.string.login_validation), Toast.LENGTH_LONG).show();
                            }
                        }
                    });
        }
    }

    private void sendToListadoAnuncios() {
        Intent i = new Intent(getActivity(), ListadoAnunciosActivity.class);
        startActivity(i);
    }

    private void validate() {
        mValidate = true;
        inputLayoutEmail.setError(validateEmail(inputEmailEditText.getText().toString()));
        inputLayoutPassword.setError(validatePassword(inputPasswordEditText.getText().toString()));
    }

    public String validateEmail(String email) {
        if (email.isEmpty() || !android.util.Patterns.EMAIL_ADDRESS.matcher(email).matches()) {
            mValidate = false;
            return getActivity().getResources().getString(R.string.validation_email);
        } else {
            return "";
        }
    }

    public String validatePassword(String password) {
        if (password.isEmpty() || password.length() < 6) {
            mValidate = false;
            return getActivity().getResources().getString(R.string.validation_password);
        } else {
            return "";
        }
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (resultCode != Activity.RESULT_OK) {
            return;
        }
        if (requestCode == REQUEST_USER) {
            inputEmailEditText.setText(data.getStringExtra(RegistrarFragment.EXTRA_EMAIL));
            inputPasswordEditText.setText(data.getStringExtra(RegistrarFragment.EXTRA_PASSWORD));
        }
    }
}
