package com.example.xavierferraronsserra.sobreruedas;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.location.Location;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.design.widget.TextInputEditText;
import android.support.design.widget.TextInputLayout;
import android.support.v4.app.Fragment;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.view.inputmethod.InputMethodManager;
import android.widget.CheckBox;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.Toast;

import com.example.xavierferraronsserra.sobreruedas.Model.Agrupation;
import com.example.xavierferraronsserra.sobreruedas.Model.Usuario;
import com.example.xavierferraronsserra.sobreruedas.Utils.StorageManagement;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.auth.AuthResult;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.firestore.DocumentSnapshot;
import com.google.firebase.firestore.GeoPoint;
import com.google.firebase.firestore.QuerySnapshot;

import java.util.ArrayList;
import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;

import static android.content.ContentValues.TAG;
import static android.view.View.GONE;
import static com.example.xavierferraronsserra.sobreruedas.AltaAnuncioFragment.hideSoftKeyboard;

/**
 * Created by xavierferraronsserra on 31/3/18.
 */

public class RegistrarFragment extends Fragment {

    //Identifies the request address
    private static final int REQUEST_ADDRESS = 1;
    //Extras we send to initialize login email and password when we finish the registration process
    public static final String EXTRA_EMAIL = "com.example.xavierferraronsserra.sobreruedas.email";
    public static final String EXTRA_PASSWORD = "com.example.xavierferraronsserra.sobreruedas.password";
    //User data we use to identificate the user we are editing
    public static final String ARG_USUARIO = "com.example.xavierferraronsserra.sobreruedas.usuario";

    private FirebaseAuth mAuth = FirebaseAuth.getInstance();
    private boolean mValidate;
    private Usuario mUsuario;

    @BindView(R.id.i_register_name)
    TextInputEditText inputNameEditText;
    @BindView(R.id.il_register_name)
    TextInputLayout inputLayoutName;
    @BindView(R.id.i_register_surname)
    TextInputEditText inputSurnameEditText;
    @BindView(R.id.il_register_surname)
    TextInputLayout inputLayoutSurname;
    @BindView(R.id.i_register_email)
    TextInputEditText inputEmailEditText;
    @BindView(R.id.il_register_email)
    TextInputLayout inputLayoutEmail;
    @BindView(R.id.i_register_phone)
    TextInputEditText inputPhoneEditText;
    @BindView(R.id.il_register_phone)
    TextInputLayout inputLayoutPhone;
    @BindView(R.id.i_register_address)
    TextInputEditText inputAddressEditText;
    @BindView(R.id.il_register_address)
    TextInputLayout inputLayoutAddress;
    @BindView(R.id.i_register_password)
    TextInputEditText inputPasswordEditText;
    @BindView(R.id.il_register_password)
    TextInputLayout inputLayoutPassword;
    @BindView(R.id.i_register_repassword)
    TextInputEditText inputRepasswordEditText;
    @BindView(R.id.il_register_repassword)
    TextInputLayout inputLayoutRepassword;
    @BindView(R.id.register_cb_xat)
    CheckBox cbXat;
    @BindView(R.id.register_cb_phone)
    CheckBox cbPhone;
    @BindView(R.id.register_cb_email)
    CheckBox cbEmail;
    @BindView(R.id.register_cb_whatsapp)
    CheckBox cbWhatsapp;
    @BindView(R.id.register_ly_passwords)
    LinearLayout  mLinearLayoutPasswords;
    @BindView(R.id.register_ly_whatsapp)
    LinearLayout  mLinearLayoutWhatsapp;
    @BindView(R.id.registrar_progress_bar)
    ProgressBar mProgressBar;

    public static RegistrarFragment newInstance(Usuario usuario) {
        Bundle args = new Bundle();
        args.putParcelable(ARG_USUARIO, usuario);
        RegistrarFragment registrarFragment = new RegistrarFragment();
        registrarFragment.setArguments(args);
        return registrarFragment;
    }

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setHasOptionsMenu(true);
        mUsuario = new Usuario();
        //By default user has contact by email activated
        mUsuario.setContactoEmail(true);
        //We check if we are creating a new user or we are editing and existing profile
        if (getArguments() != null) {
            mUsuario = (Usuario) getArguments().getParcelable(ARG_USUARIO);
        }
    }

    @Nullable
    @Override
    public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        final View view = inflater.inflate(R.layout.fragment_registrar, container, false);
        ButterKnife.bind(this, view);

        // Hide keyboard when touch on screen
        view.setOnTouchListener(new View.OnTouchListener() {
            @Override
            public boolean onTouch(View v, MotionEvent event) {
                hideSoftKeyboard(view, getActivity());
                return true;
            }
        });

        //Whatsapp not implemented
        mLinearLayoutWhatsapp.setVisibility(GONE);
        //If we are in editing mode we hide password fields and disable email field becouse is the user key identification
        if (mUsuario.getId() != null) {
            mLinearLayoutPasswords.setVisibility(GONE);
            inputEmailEditText.setEnabled(false);
        }
        //Activate validation control
        setOnFocusChange();
        //Controls if validation is ok
        mValidate = true;
        //Initialize form fields
        initializeForm();
        return view;
    }

    public void setOnFocusChange() {
        inputAddressEditText.setOnFocusChangeListener(new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View v, boolean hasFocus) {
                if (hasFocus) {
                    askForAddress();
                }
            }
        });
        inputNameEditText.setOnFocusChangeListener(new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View v, boolean hasFocus) {
                if (!hasFocus) {
                    inputLayoutName.setError(validateNotNull(inputNameEditText.getText().toString()));
                }
            }
        });
        inputSurnameEditText.setOnFocusChangeListener(new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View v, boolean hasFocus) {
                if (!hasFocus) {
                    inputLayoutSurname.setError(validateNotNull(inputSurnameEditText.getText().toString()));
                }
            }
        });
        inputEmailEditText.setOnFocusChangeListener(new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View v, boolean hasFocus) {
                if (!hasFocus) {
                    inputLayoutEmail.setError(validateEmail(inputEmailEditText.getText().toString()));
                }
            }
        });
        inputPhoneEditText.setOnFocusChangeListener(new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View v, boolean hasFocus) {
                if (!hasFocus) {
                    inputLayoutPhone.setError(validateNotNull(inputPhoneEditText.getText().toString()));
                }
            }
        });
        inputPasswordEditText.setOnFocusChangeListener(new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View v, boolean hasFocus) {
                if (!hasFocus) {
                    inputLayoutPassword.setError(validatePassword(inputPasswordEditText.getText().toString()));
                }
            }
        });
        inputRepasswordEditText.setOnFocusChangeListener(new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View v, boolean hasFocus) {
                if (!hasFocus) {
                    inputLayoutRepassword.setError(validateRepassword(inputPasswordEditText.getText().toString(), inputRepasswordEditText.getText().toString()));
                }
            }
        });
    }

    @Override
    public void onCreateOptionsMenu(Menu menu, MenuInflater inflater) {
        super.onCreateOptionsMenu(menu, inflater);
        inflater.inflate(R.menu.menu_save, menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.menu_save:
                mProgressBar.setVisibility(View.VISIBLE);
                getActivity().getWindow().setFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE, WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);
                mValidate = true;
                validate();
                if (mValidate) {
                    //We update mUsuario with form data
                    setUsuario();
                    if (mUsuario.getId() == null) {
                        //We create new user
                        newUsuario();
                    } else {
                        //We update user data
                        updateUsuario();
                    }
                }
                else{
                    mProgressBar.setVisibility(GONE);
                    getActivity().getWindow().clearFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);
                }
                return true;

            default:
                return super.onOptionsItemSelected(item);
        }
    }

    public static void hideSoftKeyboard(View view, Activity activity) {
        InputMethodManager in = (InputMethodManager) activity.getSystemService(Context.INPUT_METHOD_SERVICE);
        in.hideSoftInputFromWindow(view.getWindowToken(), InputMethodManager.HIDE_NOT_ALWAYS);
    }


    private void sendLogin() {
        Intent i = new Intent();
        i.putExtra(EXTRA_EMAIL, mUsuario.getEmail());
        i.putExtra(EXTRA_PASSWORD, inputPasswordEditText.getText().toString());
        getActivity().setResult(Activity.RESULT_OK, i);
        getActivity().finish();
    }

    private void sendListadoAnuncios() {
        Intent i = new Intent(getContext(), ListadoAnunciosActivity.class);
        startActivity(i);
    }

    private void initializeForm() {
        inputNameEditText.setText(mUsuario.getNombre());
        inputSurnameEditText.setText(mUsuario.getApellidos());
        inputPhoneEditText.setText(mUsuario.getTelefono());
        inputEmailEditText.setText(mUsuario.getEmail());
        inputAddressEditText.setText(mUsuario.getDireccion());
        cbEmail.setChecked(mUsuario.isContactoEmail());
        cbPhone.setChecked(mUsuario.isContactoTelefono());
        cbWhatsapp.setChecked(mUsuario.isContactoWhatsapp());
        cbXat.setChecked(mUsuario.isContactoChat());
    }

    private void setUsuario() {
        mUsuario.setNombre(inputNameEditText.getText().toString());
        mUsuario.setApellidos(inputSurnameEditText.getText().toString());
        mUsuario.setEmail(inputEmailEditText.getText().toString());
        mUsuario.setTelefono(inputPhoneEditText.getText().toString());
        mUsuario.setDireccion(inputAddressEditText.getText().toString());
        mUsuario.setContactoTelefono(cbPhone.isChecked());
        mUsuario.setContactoChat(cbXat.isChecked());
        mUsuario.setContactoWhatsapp(cbWhatsapp.isChecked());
        mUsuario.setContactoEmail(cbEmail.isChecked());
        mUsuario.setDireccion(inputAddressEditText.getText().toString());
    }

    @OnClick(R.id.i_register_address)
    public void askForAddress() {
        double[] location = null;
        //If user has some location data we pass it to the map
        if (mUsuario.getPosicion() != null) {
            location=new double[2];
            location[0] = mUsuario.getPosicion().getLatitude();
            location[1] = mUsuario.getPosicion().getLongitude();
        }
        Intent i = InformarDireccionActivity.newIntent(getActivity(), location);
        new Intent(getContext(), InformarDireccionActivity.class);
        startActivityForResult(i, REQUEST_ADDRESS);
    }

    private boolean validate() {

        mValidate = true;
        inputLayoutEmail.setError(validateEmail(inputEmailEditText.getText().toString()));
        inputLayoutName.setError(validateNotNull(inputNameEditText.getText().toString()));
        inputLayoutSurname.setError(validateNotNull(inputSurnameEditText.getText().toString()));
        inputLayoutPhone.setError(validateNotNull(inputPhoneEditText.getText().toString()));
        //If we are creating a new user we validate password fields
        if (mUsuario.getId() == null) {
            inputLayoutPassword.setError(validatePassword(inputPasswordEditText.getText().toString()));
            inputLayoutRepassword.setError(validateRepassword(inputPasswordEditText.getText().toString(), inputRepasswordEditText.getText().toString()));
        }
        validateCheckBoxNull(cbEmail, cbPhone, cbWhatsapp, cbXat);
        return mValidate;
    }

    private String validateNotNull(String s) {
        if (s.isEmpty()) {
            mValidate = false;
            return getActivity().getResources().getString(R.string.validation_not_null);
        } else {
            return "";
        }
    }

    public String validateEmail(String email) {
        if (email.isEmpty() || !android.util.Patterns.EMAIL_ADDRESS.matcher(email).matches()) {
            mValidate = false;
            return getActivity().getResources().getString(R.string.validation_email);
        } else {
            return "";
        }
    }

    public String validatePassword(String password) {
        if (password.isEmpty() || password.length() < 6) {
            mValidate = false;
            return getActivity().getResources().getString(R.string.validation_password);
        } else {
            return "";
        }
    }

    public String validateRepassword(String password, String repassword) {
        if (!(password.equals(repassword))) {
            mValidate = false;
            return getActivity().getResources().getString(R.string.validation_repassword);
        } else {
            return "";
        }
    }

    public void validateCheckBoxNull(CheckBox cb1, CheckBox cb2, CheckBox cb3, CheckBox cb4) {
        if (!(cb1.isChecked() || cb2.isChecked() || cb3.isChecked() || cb4.isChecked())) {
            mValidate = false;
            Toast.makeText(getContext(), getActivity().getResources().getString(R.string.validation_contact_form), Toast.LENGTH_LONG).show();
        }
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (resultCode != Activity.RESULT_OK) {
            return;
        }
        if (requestCode == REQUEST_ADDRESS) {
            Location location = (Location) data.getParcelableExtra(InformarDireccionFragment.EXTRA_LOCATION);
            mUsuario.setPosicion(new GeoPoint(location.getLatitude(), location.getLongitude()));
            String address = data.getStringExtra(InformarDireccionFragment.EXTRA_ADDRESS);
            inputAddressEditText.setText(address);
        }
    }

    public void updateUsuario() {
        StorageManagement.updateUsuario(mUsuario.getId())
                .set(mUsuario)
                .addOnCompleteListener(new OnCompleteListener<Void>() {
                    @Override
                    public void onComplete(@NonNull Task<Void> task) {
                        mProgressBar.setVisibility(GONE);
                        getActivity().getWindow().clearFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);
                        if (task.isSuccessful()) {
                            Log.d(TAG, "update usuario:ok");
                            sendListadoAnuncios();
                        } else {
                            Log.w(TAG, "Update usuario failure.");
                        }
                    }
                });
    }

    public void newUsuario() {
        mAuth.createUserWithEmailAndPassword(inputEmailEditText.getText().toString(), inputPasswordEditText.getText().toString())
                .addOnCompleteListener(getActivity(), new OnCompleteListener<AuthResult>() {
                    @Override
                    public void onComplete(@NonNull Task<AuthResult> task) {
                        if (task.isSuccessful()) {
                            Log.d(TAG, "createUserWithEmail:ok");
                            mUsuario.setId(task.getResult().getUser().getUid().toString());
                            final List<Agrupation> agrupations = new ArrayList<>();
                            //We get agrupations in order to create user subscriptions
                            StorageManagement.getAgrupations(getContext())
                                    .get()
                                    .addOnCompleteListener(new OnCompleteListener<QuerySnapshot>() {
                                        @Override
                                        public void onComplete(@NonNull Task<QuerySnapshot> task) {
                                            if (task.isSuccessful()) {
                                                for (DocumentSnapshot document : task.getResult()) {
                                                    Agrupation agrupation = document.toObject(Agrupation.class);
                                                    agrupation.setId(document.getId());
                                                    agrupations.add(agrupation);
                                                    Log.d(TAG, document.getId() + " => " + document.toObject(Agrupation.class).toString());
                                                }
                                                //Create new user
                                                StorageManagement.newUsuario(getContext(), mUsuario, agrupations)
                                                        .addOnCompleteListener(new OnCompleteListener<Void>() {
                                                            @Override
                                                            public void onComplete(@NonNull Task<Void> task) {
                                                                mProgressBar.setVisibility(GONE);
                                                                getActivity().getWindow().clearFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);
                                                                if (task.isSuccessful()) {
                                                                    Log.d(TAG, "New User transaction success!");
                                                                    sendLogin();
                                                                } else {
                                                                    Log.w(TAG, "New User transaction failure.");
                                                                }
                                                            }
                                                        });
                                            } else {
                                                Log.w(TAG, "Getting Agrupations New User failure.");
                                            }
                                        }
                                    });
                        } else {
                            mProgressBar.setVisibility(GONE);
                            getActivity().getWindow().clearFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);
                            Toast.makeText(getContext(), getResources().getString(R.string.register_error) + task.getException().toString(), Toast.LENGTH_LONG).show();
                            Log.e(TAG, "createUserWithEmail:error: " + task.getException().toString());
                        }
                    }
                });
    }
}
