package com.example.xavierferraronsserra.sobreruedas.Utils;

import android.content.Context;
import android.graphics.Bitmap;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.util.Log;

import com.algolia.search.saas.AbstractQuery;
import com.algolia.search.saas.AlgoliaException;
import com.algolia.search.saas.Client;
import com.algolia.search.saas.CompletionHandler;
import com.algolia.search.saas.Index;
import com.example.xavierferraronsserra.sobreruedas.FilterFragment;
import com.example.xavierferraronsserra.sobreruedas.Model.Agrupation;
import com.example.xavierferraronsserra.sobreruedas.Model.Anuncio;
import com.example.xavierferraronsserra.sobreruedas.Model.Filter;
import com.example.xavierferraronsserra.sobreruedas.Model.Token;
import com.example.xavierferraronsserra.sobreruedas.Model.Usuario;
import com.example.xavierferraronsserra.sobreruedas.R;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.OnFailureListener;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.firestore.CollectionReference;
import com.google.firebase.firestore.DocumentReference;
import com.google.firebase.firestore.DocumentSnapshot;
import com.google.firebase.firestore.FirebaseFirestore;
import com.google.firebase.firestore.FirebaseFirestoreException;
import com.google.firebase.firestore.Query;
import com.google.firebase.firestore.QuerySnapshot;
import com.google.firebase.firestore.Transaction;
import com.google.firebase.firestore.WriteBatch;
import com.google.firebase.messaging.FirebaseMessaging;
import com.google.firebase.storage.FirebaseStorage;
import com.google.firebase.storage.StorageReference;
import com.google.firebase.storage.UploadTask;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.ByteArrayOutputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * Created by xavierferraronsserra on 5/4/18.
 */

public class StorageManagement {

    private static final String TAG = "StorageManagement";

    //Firebase data
    private static final String USUARIOS = "Usuarios";
    private static final String AGRUPACIONES = "Agrupaciones";
    private static final String ANUNCIOS = "Anuncios";
    private static final String FOTOS = "Fotos";
    private static final String NOTIFICACIONES = "Notificaciones";
    private static final String TOKENS = "Tokens";

    //Algolia data
    private static final String APP_ID = "FCRL3BY4F4";
    private static final String KEY = "5192de33eb3ebc1e29b33923c6a4e68a";
    private static final String INDEX_NAME = "anuncios";
    private static final int PAGE_ITEMS = 15;
    private static final int MAX_ITEMS = 50;

    private static final StorageManagement firebase = new StorageManagement();

    //Firebase instances
    private static FirebaseAuth mAuth = FirebaseAuth.getInstance();
    private static FirebaseFirestore mDatabase = FirebaseFirestore.getInstance();
    private static FirebaseStorage mStorage = FirebaseStorage.getInstance();
    private static FirebaseMessaging mFCM = FirebaseMessaging.getInstance();

    //Algolia index
    private static Client client = new Client(APP_ID, KEY);
    private static Index mIndex = client.getIndex(INDEX_NAME);

    private StorageManagement() {
    }

    public static StorageManagement getInstance() {
        return firebase;
    }

    public static Index getIndex() {
        return mIndex;
    }

    public static int getPageItems() {
        return PAGE_ITEMS;
    }

    public static int getMaxItems() {
        return MAX_ITEMS;
    }

    public static DocumentReference getToken(Context context, String userId, String token) {
        return mDatabase.collection(USUARIOS).document(userId).collection(TOKENS).document(token);
    }

    public static void newToken(Context context, String userId, final String tokenId) {
        final Token token = new Token(tokenId, userId);
        final DocumentReference userTokenRef = mDatabase.collection(USUARIOS).document(userId).collection(TOKENS).document(tokenId);

        //We check which subscriptions the user has active.
        Query userActiveNotifQuery = mDatabase.collection(USUARIOS).document(userId).collection(NOTIFICACIONES).whereEqualTo("active", true);
        userActiveNotifQuery.get().addOnCompleteListener(new OnCompleteListener<QuerySnapshot>() {
            @Override
            public void onComplete(@NonNull Task<QuerySnapshot> task) {
                if (task.isSuccessful()) {
                    final List<Agrupation> activeNotifications = new ArrayList<>();
                    for (DocumentSnapshot document : task.getResult()) {
                        activeNotifications.add(document.toObject(Agrupation.class));
                    }
                    //In the active subscriptions we register new token
                    if (activeNotifications.size() > 0) {
                        mDatabase.runTransaction(new Transaction.Function<Void>() {
                            @Override
                            public Void apply(Transaction transaction) throws FirebaseFirestoreException {
                                for (int i = 0; i < activeNotifications.size(); i++) {
                                    Agrupation agrupation = activeNotifications.get(i);
                                    transaction.set(mDatabase.collection(AGRUPACIONES).document(agrupation.getId()).collection(TOKENS).document(tokenId), token);
                                }
                                //In the same transaction we register token to the user
                                transaction.set(userTokenRef, token);
                                return null;
                            }
                        }).addOnSuccessListener(new OnSuccessListener<Void>() {
                            @Override
                            public void onSuccess(Void aVoid) {
                                Log.d(TAG, "Transaction new token success!");
                            }
                        }).addOnFailureListener(new OnFailureListener() {
                            @Override
                            public void onFailure(@NonNull Exception e) {
                                Log.w(TAG, "Transaction new token failure.", e);
                            }
                        });
                    }
                    //If there is not active subscriptions we register token to the user
                    else {
                        userTokenRef.set(token).addOnCompleteListener(new OnCompleteListener<Void>() {
                            @Override
                            public void onComplete(@NonNull Task<Void> task) {
                                if (task.isSuccessful()) {
                                    Log.d(TAG, "Transaction new token success!");
                                } else {
                                    Log.w(TAG, "Transaction new token failure.");
                                }
                            }
                        });
                    }

                }
            }
        });

    }

    public static Query getUserNotifications(String userId) {
        return mDatabase.collection(USUARIOS).document(userId).collection(NOTIFICACIONES).orderBy("descripcion");
    }

    public static Task newImage(String anuncioId, String posicion, Bitmap bitmap, int action) {

        StorageReference imageRef = mStorage.getReference().child(anuncioId).child(posicion);

        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        bitmap.compress(Bitmap.CompressFormat.JPEG, 100, baos);
        byte[] data = baos.toByteArray();

        UploadTask uploadTask = imageRef.putBytes(data);
        return uploadTask;
    }

    public static Task deleteImage(String anuncioId, String posicion) {
        return mStorage.getReference().child(anuncioId).child(posicion).delete();
    }

    public static void setUserNotification(Context context, final String userId, final String notificationId, final boolean active) {
        final DocumentReference notificationRef = mDatabase.collection(USUARIOS).document(userId).collection(NOTIFICACIONES).document(notificationId);
        final CollectionReference userTokensRef = mDatabase.collection(USUARIOS).document(userId).collection(TOKENS);
        final CollectionReference tokensAgrupationRef = mDatabase.collection(AGRUPACIONES).document(notificationId).collection(TOKENS);

        //we get all user tokens
        userTokensRef.get().addOnCompleteListener(new OnCompleteListener<QuerySnapshot>() {
            @Override
            public void onComplete(@NonNull Task<QuerySnapshot> task) {
                if (task.isSuccessful()) {
                    List<Token> tokens = new ArrayList<>();
                    tokens = task.getResult().toObjects(Token.class);
                    final List<Token> finalTokens = tokens;
                    //when we have all tokens we create a transaction in order to be sure we do all changes together and don't leave tokens with errors
                    mDatabase.runTransaction(new Transaction.Function<Void>() {
                        @Override
                        public Void apply(final Transaction transaction) throws FirebaseFirestoreException {
                            transaction.update(notificationRef, "active", active);
                            for (int i = 0; i < finalTokens.size(); i++) {
                                Token token = finalTokens.get(i);
                                //if we activate notification we add tokens to the agrupation
                                if (active) {
                                    transaction.set(tokensAgrupationRef.document(token.getToken()), token);
                                }
                                //if we desactivate notification we delete tokens of the agrupation
                                if (!active) {
                                    transaction.delete(tokensAgrupationRef.document(token.getToken()));
                                }
                            }
                            return null;
                        }
                    }).addOnSuccessListener(new OnSuccessListener<Void>() {
                        @Override
                        public void onSuccess(Void aVoid) {
                            Log.d(TAG, "Updated user tokens in agrupation");
                        }
                    })
                            .addOnFailureListener(new OnFailureListener() {
                                @Override
                                public void onFailure(@NonNull Exception e) {
                                    Log.w(TAG, "User tokens updated in agrupation failure.", e);
                                }
                            });
                } else {
                    Log.w(TAG, "Error getting tokens to update agrupations");
                }
            }
        });
    }

    public static DocumentReference getAgrupationById(String id) {

        return mDatabase.collection(AGRUPACIONES).document(id);
    }

    public static DocumentReference getAnuncioById(String id) {

        return mDatabase.collection(ANUNCIOS).document(id);
    }

    public static void incrementVisits(final String anuncioId) {
        mDatabase.collection(ANUNCIOS).document(anuncioId).get()
                .addOnCompleteListener(new OnCompleteListener<DocumentSnapshot>() {
                    @Override
                    public void onComplete(@NonNull Task<DocumentSnapshot> task) {
                        if (task.isSuccessful()) {
                            final Anuncio anuncio = task.getResult().toObject(Anuncio.class);
                            Map<String, Object> data = new HashMap<>();
                            data.put("visitas", anuncio.getVisitas() + 1);
                            mDatabase.collection(ANUNCIOS).document(anuncioId).update(data).addOnCompleteListener(new OnCompleteListener<Void>() {
                                @Override
                                public void onComplete(@NonNull Task<Void> task) {
                                    if (task.isSuccessful()) {
                                        Log.d(TAG, "Incremented successfully number of visits");
                                        StorageManagement.getIndex().saveObjectAsync(Utils.ObjectToJsonAnuncio(anuncio), anuncio.getId(), null);
                                    } else {
                                        Log.w(TAG, "Failure incremneting number of visits");
                                    }
                                }
                            });
                        }
                    }
                });
    }

    public static Task<Void> newUsuario(Context context, final Usuario usuario, final List<Agrupation> agrupations) {
        return mDatabase.runTransaction(new Transaction.Function<Void>() {
            @Nullable
            @Override
            public Void apply(@NonNull Transaction transaction) throws FirebaseFirestoreException {

                transaction.set(mDatabase.collection(USUARIOS).document(mAuth.getCurrentUser().getUid().toString()), usuario);
                for (int i = 0; i < agrupations.size(); i++) {
                    Agrupation agrupation = agrupations.get(i);
                    transaction.set(mDatabase.collection(USUARIOS).document(mAuth.getCurrentUser().getUid().toString()).collection(NOTIFICACIONES).document(agrupation.getId()), agrupation);
                }
                return null;
            }
        });
    }

    public static DocumentReference updateUsuario(String userId) {
        return mDatabase.collection(USUARIOS).document(userId);
    }

    public static DocumentReference getUser(Context context, String userId) {
        return mDatabase.collection(USUARIOS).document(userId);
    }

    public static com.algolia.search.saas.Query getAnunciosAlgolia(Context context, Filter filter) {
        com.algolia.search.saas.Query query = new com.algolia.search.saas.Query(filter.getDescription());
        String condicion = "";
        condicion = "activo=1";

        if (!filter.getAgrupation().equals("0")) {
            condicion = condicion + " AND agrupacion: " + filter.getAgrupation();
        }

        if (filter.getDistSup() != FilterFragment.getMaxDistance() && filter.getPosition().getLatitude() != 0 && filter.getPosition().getLongitude() != 0) {
            query.setAroundRadius(filter.getDistSup() * 1000);
            query.setAroundLatLng(new AbstractQuery.LatLng(filter.getPosition().getLatitude(), filter.getPosition().getLongitude()));
        }
        if (filter.getPriceInf() != 0) {
            condicion = condicion + " AND price >=" + filter.getPriceInf();
        }
        if (filter.getPriceSup() != FilterFragment.getMaxPrice()) {
            condicion = condicion + " AND price<=" + filter.getPriceSup();
        }
        query.setFilters(condicion);

        return query;
    }

    public static Query getAnunciosUsuario(Context context, String userId) {
        Query anunciosRef = mDatabase.collection(ANUNCIOS).whereEqualTo("id_usuario", mAuth.getCurrentUser().getUid()).orderBy("fecha_alta", Query.Direction.DESCENDING);

        return anunciosRef;
    }

    public static Task<Void> newAnuncio(final Anuncio anuncio) {
        return mDatabase.collection(ANUNCIOS).document(anuncio.getId()).set(anuncio);

    }

    public static Query getAgrupations(Context context) {
        return mDatabase.collection(AGRUPACIONES).orderBy("descripcion", Query.Direction.ASCENDING);
    }

    public static void createAgrupations(Context context) {
        Agrupation agrupation1 = new Agrupation(context.getResources().getString(R.string.wheelchairs));
        Agrupation agrupation2 = new Agrupation(context.getResources().getString(R.string.assisted_wheelchairs));
        Agrupation agrupation3 = new Agrupation(context.getResources().getString(R.string.sport_wheelchairs));
        Agrupation agrupation4 = new Agrupation(context.getResources().getString(R.string.scooters));
        Agrupation agrupation5 = new Agrupation(context.getResources().getString(R.string.urban_handbikes));
        Agrupation agrupation6 = new Agrupation(context.getResources().getString(R.string.sport_handbikes));
        Agrupation agrupation7 = new Agrupation(context.getResources().getString(R.string.driver_adapted_vehicle));
        Agrupation agrupation8 = new Agrupation(context.getResources().getString(R.string.companion_adapted_vehicle));
        Agrupation agrupation9 = new Agrupation(context.getResources().getString(R.string.home_helps));
        Agrupation agrupation10 = new Agrupation(context.getResources().getString(R.string.rehabilitation));
        Agrupation agrupation11 = new Agrupation(context.getResources().getString(R.string.accessories));

        WriteBatch batch = mDatabase.batch();
        CollectionReference agrupationsRef = mDatabase.collection(AGRUPACIONES);

        mDatabase.collection(AGRUPACIONES).add(agrupation1);
        mDatabase.collection(AGRUPACIONES).add(agrupation2);
        mDatabase.collection(AGRUPACIONES).add(agrupation3);
        mDatabase.collection(AGRUPACIONES).add(agrupation4);
        mDatabase.collection(AGRUPACIONES).add(agrupation5);
        mDatabase.collection(AGRUPACIONES).add(agrupation6);
        mDatabase.collection(AGRUPACIONES).add(agrupation7);
        mDatabase.collection(AGRUPACIONES).add(agrupation8);
        mDatabase.collection(AGRUPACIONES).add(agrupation9);
        mDatabase.collection(AGRUPACIONES).add(agrupation10);
        mDatabase.collection(AGRUPACIONES).add(agrupation11);

    }

    public static void createTopics(Context context) {

        getAgrupations(context).get().addOnCompleteListener(new OnCompleteListener<QuerySnapshot>() {
            @Override
            public void onComplete(@NonNull Task<QuerySnapshot> task) {
                if (task.isSuccessful()) {
                    for (DocumentSnapshot document : task.getResult()) {
                        Agrupation agrupation = document.toObject(Agrupation.class);
                        agrupation.setId(document.getId());
                        mFCM.subscribeToTopic(agrupation.getId());
                        Log.d(TAG, document.getId() + " => " + document.toObject(Agrupation.class).toString());
                    }
                }
            }
        });


    }

    public static void deleteAnuncios(final HashMap<String, Boolean> selectedItems) throws JSONException {
        final List<String> array = new ArrayList<>();
        WriteBatch batch = mDatabase.batch();
        Iterator<String> it = selectedItems.keySet().iterator();
        while (it.hasNext()) {
            String key = (String) it.next();
            DocumentReference docRef = mDatabase.collection(ANUNCIOS).document(key);
            batch.delete(docRef);
            array.add(key);
            StorageReference imagesRef = mStorage.getReference().child(key);
            for (int i = 0; i < 4; i++) {
                imagesRef.child(Integer.toString(i))
                        .delete()
                        .addOnSuccessListener(new OnSuccessListener<Void>() {
                            @Override
                            public void onSuccess(Void aVoid) {
                                Log.d(TAG, "Image successfully deleted");
                            }
                        }).addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception exception) {
                        Log.w(TAG, "Error deleting image: " + exception.toString());
                    }
                });

            }
        }
        batch.commit().addOnCompleteListener(new OnCompleteListener<Void>() {
            @Override
            public void onComplete(@NonNull Task<Void> task) {
                if (task.isSuccessful()) {
                    Log.d(TAG, "Deleted product successfully");
                    getIndex().deleteObjectsAsync(array, new CompletionHandler() {
                        @Override
                        public void requestCompleted(JSONObject jsonObject, AlgoliaException e) {
                            if (e == null) {
                                Log.d(TAG, "Deleted product successfully in Algolia");
                            } else {
                                Log.w(TAG, "Deleting product failure in algolia", e);
                            }
                        }
                    });
                } else {
                    Log.w(TAG, "Deleting product failure", task.getException());
                }
            }
        });
    }

    public static void changeStateAnuncios(final HashMap<String, Boolean> selectedItems) throws JSONException {
        final List<JSONObject> array = new ArrayList<JSONObject>();
        WriteBatch batch = mDatabase.batch();
        Iterator<String> it = selectedItems.keySet().iterator();
        while (it.hasNext()) {
            String key = (String) it.next();
            boolean value = !selectedItems.get(key);
            DocumentReference docRef = mDatabase.collection(ANUNCIOS).document(key);
            batch.update(docRef, "activo", value);
            if (value == false) {
                array.add(new JSONObject().put("activo", value).put("objectID", key));
            } else {
                batch.update(docRef, "fecha_renovacion", new Date());
                array.add(new JSONObject().put("activo", value).put("objectID", key).put("fecha_renovacion", new Date().getTime()/1000));
            }
        }
        batch.commit().addOnCompleteListener(new OnCompleteListener<Void>() {
            @Override
            public void onComplete(@NonNull Task<Void> task) {
                if (task.isSuccessful()) {
                    Log.d(TAG, "Changed state successfully");
                    getIndex().partialUpdateObjectsAsync(new JSONArray(array), false, new CompletionHandler() {
                        @Override
                        public void requestCompleted(JSONObject jsonObject, AlgoliaException e) {
                            if (e == null) {
                                Log.d(TAG, "Changed state successfully in Algolia");
                            } else {
                                Log.w(TAG, "State Change failure in algolia", e);

                            }
                        }
                    });
                } else {
                    Log.w(TAG, "State Change failure.", task.getException());
                }
            }
        });

    }


}