"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// The Cloud Functions for Firebase SDK to create Cloud Functions and setup triggers.
const functions = require("firebase-functions");
const TOKENS_AGRUP = "Agrupaciones/{agrupId}/Tokens/{tokenId}";
const NEW_PRODUCT = "Anuncios/{anuncioId}";
const PRODUCTS = "Anuncios";
// The Firebase Admin SDK to access the Firebase Realtime Database.
const admin = require('firebase-admin');
admin.initializeApp(functions.config().firebase);
exports.subscribeToTopic = functions.firestore.document(TOKENS_AGRUP)
    .onCreate(event => {
    const token = event.data.data();
    console.log('Added token to agrupation', event.params.tokenId, event.params.agrupId);
    admin.messaging().subscribeToTopic(event.params.tokenId, event.params.agrupId)
        .then(function (response) {
        console.log("Successfully subscribed to topic: ", response);
    })
        .catch(function (error) {
        console.log("Error subscribing to topic: ", error);
    });
});
exports.sendToTopic = functions.firestore.document(NEW_PRODUCT)
    .onCreate(event => {
    const anuncio = event.data.data();
    console.log('Added anuncio', event.params.anuncioId, anuncio.price, anuncio.descripcion);
    const payLoad = {
        notification: {
            title: 'Añadido el anuncio: ' + anuncio.title + ' (' + anuncio.price + '€)',
            body: anuncio.descripcion,
        },
        data: {
            anuncioId: event.params.anuncioId
        }
    };
    const options = {
        priority: "high",
        timeToLive: 60 * 60 * 2
    };
    admin.messaging().sendToTopic(anuncio.agrupacion, payLoad, options)
        .then(function (response) {
        console.log("Successfully sended message to topic: ", response, anuncio.agrupacion);
    })
        .catch(function (error) {
        console.log("Error sending message to topic: ", error);
    });
});
exports.caducity = functions.https.onRequest((req, res) => {
    const today = new Date();
    const date = new Date();
    date.setDate(today.getDate() - 60);
    admin.firestore().collection(PRODUCTS)
        .where("activo", "==", true)
        .get()
        .then(function (querySnapshot) {
        querySnapshot.forEach(function (doc) {
            // doc.data() is never undefined for query doc snapshots
            console.log(doc.id, " => ", doc.data());
            console.log(doc.ref);
            doc.ref.update({
                capital: true
            })
                .then(function () {
                console.log("Document successfully updated!");
            })
                .catch(function (error) {
                // The document probably doesn't exist.
                console.error("Error updating document: ", error);
            });
        });
    })
        .catch(function (error) {
        console.log("Error getting documents: ", error);
    });
    return res;
});
//# sourceMappingURL=a.js.map