"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// The Cloud Functions for Firebase SDK to create Cloud Functions and setup triggers.
const functions = require("firebase-functions");
const TOKENS_AGRUP = "Agrupaciones/{agrupId}/Tokens/{tokenId}";
const TOKENS_USUARIO = "Usuarios/{usuarioId}/Tokens/{tokenId}";
const NEW_PRODUCT = "Anuncios/{anuncioId}";
const PRODUCTS = "Anuncios";
// The Firebase Admin SDK to access the Firebase Realtime Database.
const admin = require('firebase-admin');
const algoliasearch = require('algoliasearch');
admin.initializeApp(functions.config().firebase);
exports.subscribeToTopic = functions.firestore.document(TOKENS_AGRUP)
    .onCreate(event => {
    console.log('Added token to agrupation', event.params.tokenId, event.params.agrupId);
    admin.messaging().subscribeToTopic(event.params.tokenId, event.params.agrupId)
        .then(function (response) {
        console.log("Successfully subscribed to topic: ", response);
    })
        .catch(function (error) {
        console.log("Error subscribing to topic: ", error);
    });
});
exports.subscribeToUser = functions.firestore.document(TOKENS_USUARIO)
    .onWrite(event => {
    console.log('Added token to user', event.params.tokenId, event.params.usuarioId);
    admin.messaging().subscribeToTopic(event.params.tokenId, event.params.usuarioId)
        .then(function (response) {
        console.log("Successfully subscribed to topic: ", response);
    })
        .catch(function (error) {
        console.log("Error subscribing to topic: ", error, event.params.usuarioId);
    });
});
exports.sendToTopic = functions.firestore.document(NEW_PRODUCT)
    .onCreate(event => {
    const anuncio = event.data.data();
    console.log('Added anuncio', event.params.anuncioId, anuncio.price, anuncio.descripcion);
    const payLoad = {
        notification: {
            title: 'Añadido el anuncio: ' + anuncio.title + ' (' + anuncio.price + '€)',
            body: anuncio.descripcion,
        },
        data: {
            type: 'NewProduct',
            anuncioId: event.params.anuncioId
        }
    };
    const options = {
        priority: "high",
        timeToLive: 60 * 60 * 2
    };
    admin.messaging().sendToTopic(anuncio.agrupacion, payLoad, options)
        .then(function (response) {
        console.log("Successfully sended message to topic: ", response, anuncio.agrupacion);
    })
        .catch(function (error) {
        console.log("Error sending message to topic: ", error);
    });
});
//# sourceMappingURL=index.js.map